#!/usr/bin/perl
# $Id: FIPS-AddProvenance.pl,v 1.2 2006/11/15 22:43:56 rnotestine Exp $
################################################################################
# Dependencies:
#               XML::XPath - SRB@/home/dkeator@uci-bic/NonHumanPhantomScript/XML
#
# Authors:      David Keator (dbkeator@uci.edu)
#               Syam Gadde (gadde@biac.duke.edu)
#               Doug Greve (greve@nmr.mgh.harvard.edu)
#               Randy Notestine (RNotestine@UCSD.edu)
#
# Revision History:
#   2006/05/22  Code taken from LocalHumanUpload/RemoveSeries.pl and
#               fips/FIPS-SetAnalysisLevel.pl
################################################################################

use FindBin qw($RealDir);
use lib "$RealDir";
require 'FIPS-XMLFuncs.pl';

# Check number of arguments
my $nargs = $#ARGV + 1;
if($nargs < 4) {
  print "\nUsage: FIPS-AddProvenance.pl xml_in xml_out element1 element1_value [ element2 element2_value [ ... ] ]\n\n";
  print "  Adds an additional <processStep> node to the <provenance> node with any of the following elements:\n";
  print "    \"programName\", \"programArgument\", \"version\", \"timeStamp\", \"cvs\", \"user\",\n";
  print "    \"machine\", \"hostName\", \"platform\", \"platformVersion\", \"compilerName\", \"compilerVersion\",\n";
  print "    \"libName\", and/or \"libVersion\"\n\n";
  exit(1);
}

my @element_names  = ("programName", "programArgument", "version", "timeStamp", "cvs", "user", "machine", "hostName",
                      "platform", "platformVersion", "compilerName", "compilerVersion", "libName", "libVersion");
my @element_values = ("", "", "", "", "", "", "", "", "", "", "", "", "", "");
my $optional_elements = 4;

my $xml_in      = shift @ARGV;
my $xml_out     = shift @ARGV;

if(! -e $xml_in ) {
  print "ERROR: xml input file \"$xml_in\" does not exist.\n";
  exit(1);
}

# Standard Stuff to read in an XML document
my $parser = new XML::DOM::Parser;
my $doc = $parser->parsefile("$xml_in");
my $root_node = $doc->getDocumentElement();

my $provenance_xpath = "serieslevel/provenance";
my $provenance_node = findNode($doc, $provenance_xpath);

# Add a <provenance> node if one doesn't already exist
if (! defined($provenance_node)) {
  $provenance_node = appendNodeXPath($doc, "provenance", "serieslevel");
  if (! defined($provenance_node)) {
    print "Error: \"provenance\" node could not be found or added!";
    exit (1);
  }
}

# Loop over specified processStep elements and store element values
while ($#ARGV > 0) {
  $element_name  = shift @ARGV;
  $element_value = shift @ARGV;
  $element_unmatched = 1;

  for ($index = 0; $index <= $#element_names; $index++) {
    if ($element_names[$index] eq $element_name) {
      $element_values[$index] = $element_value;
      $element_unmatched = 0;
      last;
    }
  }

  if ($element_unmatched) {
    print "Error: Unsupported <processStep> element \"$element_name\" specified\n";
    exit (1);
  }
}

# Add a new <processStep> node
$process_node = appendNodeXPath($doc, "processStep", $provenance_xpath);
if (! defined($process_node)) {
  print "Error: \"processStep\" node could not be found or added!";
  exit (1);
}

# Append new required elements with appropriate values
for ($index = 0; $index <= ($#element_names - $optional_elements); $index++) {
  $new_node = appendNode($doc, $process_node, $element_names[$index], $element_values[$index]);
  if (! defined($new_node)) {
    print "Error: Failed to add \"$element_names[$index]\" element with value of \"$element_values[$index]\"";
    exit (1);
  } else {
    print "  Set required processStep element \"$element_names[$index]\" to \"$element_values[$index]\"\n";
  }
}

# If specified, append new optional elements (in pairs) with appropriate values
for ($index = ($#element_names - $optional_elements + 1); $index < $#element_names; $index += 2) {
  if ($element_values[$index] || $element_values[$index+1]) {
    foreach $delta (0, 1) {
      $new_node = appendNode($doc, $process_node, $element_names[$index+$delta], $element_values[$index+$delta]);
      if (! defined($new_node)) {
        print "Error: Failed to add \"$element_names[$index+$delta]\" element with value of \"$element_values[$index+$delta]\"";
        exit (1);
      } else {
        print "  Set optional processStep element \"$element_names[$index+$delta]\" to \"$element_values[$index+$delta]\"\n";
      }
    }
  }
}


# Write XML object to output file
$doc->printToFile("$xml_out");

# Avoid memory leaks - cleanup circular references
$doc->dispose;

exit(0);
