#!/usr/bin/perl -w
# $Id: FIPS-GetB0Params.pl,v 1.2 2006/11/15 22:43:56 rnotestine Exp $
###############################################################################
#  This script extracts for a given site the requested site specific BO input
#  parameter from the provided B0 parameter file. The B0 parameter file 
#  contains comma separated values for BIRN siteID, Echo Spacing and TE 
#  difference. The commnadline arguments are
#  <B0-param-file> <site-ID> <tediff|esp>
#  where the type of the parameter is specified by the third argument and
#  if the parameter value exists, it is written to standard output.
#
#  Authors:      I. Burak Ozyurt (iozyurt@ucsd.edu)
#
###############################################################################

use strict;

sub usage() {
die<<EOF
 $0 <B0-param-file> <site-ID> <tediff|esp>
EOF
}
usage() unless( scalar(@ARGV) == 3);
my $b0_param_file = $ARGV[0];
my $site_id = $ARGV[1];
my $param_type = $ARGV[2];


open(IN,"$b0_param_file") or die "cannot open file:$b0_param_file :$!";
my $value = undef;
while(<IN>) {
    chomp;
    my @toks = split(/,/,$_);
    if ( $toks[0] =~ /^$site_id$/) {
	if ($param_type =~ /tediff/) {
	    $value = $toks[2];
	} elsif ($param_type =~ /esp/) {
	    $value = $toks[1];
	} else {
	    die "not a valid param type:$param_type";
	}
	last;
    }
}
close(IN);

if (defined($value)) {
    print "$value\n";
} 

