#!/usr/bin/perl -w
# $Id: FIPS-GetExcedeSize.pl,v 1.1 2006/11/09 19:57:52 rnotestine Exp $
########################################################################
#  Given an Image wrapper file in XCEDE format, extract the (grid) size 
#  of each dimension and return them in the following format
#
#  x-size y-size z-size t-size
#
# Authors:      I. Burak Ozyurt
#               Randy Notestine (RNotestine@UCSD.edu)
#
# Revision History:
#   2006/11/09  Code taken from FIPS-GetExcedeSpacing.pl
########################################################################
use strict;

sub usage() {
die<<EOF

Usage: $0 <image-wrapper-xml-file-full-path>    
EOF
}

sub extract_sizes {
    my ($fname) = @_;
    my %size_map = ();
    my $rec = undef;
    open(IN,"$fname") or die("cannot open file:$fname:$!");
    my $in_dimension = 0;
    my $dimension = undef;
    while(<IN>) {
	chomp;
	if (/\<\/dimension/) {
	    $in_dimension = 0;
	} 
	if (/\<dimension type=\"(\w)\"/) {
	    
            # last if ($1 =~ /t/); 
	    # print "$1\n";
	    $dimension = $1;
	    $in_dimension = 1;
            my $unit = ($1 =~ /t/i) ? "ms" : "mm";
             
            $rec = { SIZE => 0 };
            $size_map{$dimension} = $rec;
	    next;
	}
	if ($in_dimension) {
	    if (/\<size\>(.+)\</) {
		# print "size=$1\n";
                $rec =  $size_map{$dimension};
		$rec->{SIZE} = $1;
	    }            
	}

    }
    close(IN);
    return %size_map;
}



sub main() {
  usage() if (@ARGV != 1) ;
  my $wrapper_file = $ARGV[0];
  my %size_map = extract_sizes($wrapper_file);
  
  my @list = ();
#  my @units = ();
  my $rec = undef;
  $rec = $size_map{"x"};
  push @list, $rec->{SIZE};
#  push @units, $rec->{UNITS};

  $rec = $size_map{"y"};
  push @list, $rec->{SIZE};
#  push @units, $rec->{UNITS};

  $rec = $size_map{"z"};
  push @list, $rec->{SIZE};
#  push @units, $rec->{UNITS};

  $rec = $size_map{"t"};
  push @list, $rec->{SIZE};
#  push @units, $rec->{UNITS};

  print join(" ",@list) . "\n";
}

main();
