#!/usr/bin/perl -w
# $Id: FIPS-GetExcedeSliceOrder.pl,v 1.1 2006/11/15 22:50:26 rnotestine Exp $
########################################################################
#  Given an Image wrapper file in XCEDE format, checks the slice order
#  counts against the z-axis slices count in the wrapper file. If they
#  match returns the slice order values as a row vector.
#
#  Authors:      I. Burak Ozyurt (iozyurt@ucsd.edu)
#
########################################################################

use strict;

sub usage() {
die<<EOF

Usage: $0 <image-wrapper-xml-file-full-path> 
EOF
}



sub extract_slice_order_info {
    my ($fname) = @_;
    my $z_size = 0;
    my @sliceorders = ();
    open(IN,"$fname") or die("cannot open file:$fname:$!");
    my $in_dimension = 0;
    while(<IN>) {
	chomp;
	if (/\<\/dimension/) {
	    $in_dimension = 0;
	} 
	if (/<acqParam name=\"sliceorder\"\s+type=\"varchar\"\s*>(.+?)<\/acqParam>/) {
	    # print "$1\n";
            @sliceorders = split(/,/, $1);  
	    next;    
	}
	if (/\<dimension type=\"z\"/) {
	    $in_dimension = 1;
	    next;
	}
        if ($in_dimension) {
	    if (/\<size\>(.+)\</) {
		$z_size = $1;     
	    }	    
	}
    }
    close(IN);
    exit 1 if ( $z_size != scalar(@sliceorders));
    return @sliceorders;
}


sub main() {
    usage() if (@ARGV != 1) ;
    my $wrapper_file = $ARGV[0];  
    
    my @slice_orders = extract_slice_order_info($wrapper_file);

    print join(" ", @slice_orders) . "\n";
   
}

main();
