#!/usr/bin/perl -w
# $Id: FIPS-GetExcedeSpacing.pl,v 1.1 2006/11/09 19:57:52 rnotestine Exp $
########################################################################
#  Given an Image wrapper file in XCEDE format, extracts the spacing 
#  and unit of measure values and returns them in following format
#
#  x-space y-space z-space t-space x-units y-units z-units t-units
#  
#  The default unit for spatial dimensions is mm and for temporal 
#  dimension ms.
#
#  Authors:      I. Burak Ozyurt
########################################################################
use strict;

sub usage() {
die<<EOF

Usage: $0 <image-wrapper-xml-file-full-path>    
EOF
}

sub extract_spacings {
    my ($fname) = @_;
    my %spacing_map = ();
    my $rec = undef;
    open(IN,"$fname") or die("cannot open file:$fname:$!");
    my $in_dimension = 0;
    my $dimension = undef;
    while(<IN>) {
	chomp;
	if (/\<\/dimension/) {
	    $in_dimension = 0;
	} 
	if (/\<dimension type=\"(\w)\"/) {
	    
            # last if ($1 =~ /t/); 
	    # print "$1\n";
	    $dimension = $1;
	    $in_dimension = 1;
            my $unit = ($1 =~ /t/i) ? "ms" : "mm";
             
            $rec = { SPACING => 0,
                     UNITS =>  $unit };
            $spacing_map{$dimension} = $rec;
	    next;
	}
	if ($in_dimension) {
	    if (/\<spacing\>(.+)\</) {
		# print "spacing=$1\n";
                $rec =  $spacing_map{$dimension};
		$rec->{SPACING} = $1;
	    } elsif (/\<units\>(\w+)\</) {
		# print "direction=$1\n";
		$rec->{UNITS} = $1;
	    }            
	}

    }
    close(IN);
    return %spacing_map;
}



sub main() {
  usage() if (@ARGV != 1) ;
  my $wrapper_file = $ARGV[0];
  my %spacing_map = extract_spacings($wrapper_file);
  
  my @list = ();
  my @units = ();
  my $rec = undef;
  $rec = $spacing_map{"x"};
  push @list, $rec->{SPACING};
  push @units, $rec->{UNITS};

  $rec = $spacing_map{"y"};
  push @list, $rec->{SPACING};
  push @units, $rec->{UNITS};

  $rec = $spacing_map{"z"};
  push @list, $rec->{SPACING};
  push @units, $rec->{UNITS};

  $rec = $spacing_map{"t"};
  push @list, $rec->{SPACING};
  push @units, $rec->{UNITS};

  print join(" ",@list) ." " . join(" ", @units) . "\n";
}

main();
