#!/usr/bin/perl 
# FIPS-SetAnalysisLevel.pl
# $Id: FIPS-SetAnalysisLevel.pl,v 1.4 2006/06/29 23:03:19 rnotestine Exp $
#################################################################################
# Sets the analysis level of a series in the fips-process.xml file
#
# Dependencies:
#		XML::XPath - SRB@/home/dkeator@uci-bic/NonHumanPhantomScript/XML
#
# Authors: 	David Keator (dbkeator@uci.edu)
#		Syam Gadde (gadde@biac.duke.edu)
#               Doug Greve (greve@nmr.mgh.harvard.edu)
#		
# Revision History: Modification of XMLFuncs.pl 
###############################################################################

use FindBin qw($RealDir);
use lib "$RealDir";
require 'FIPS-XMLFuncs.pl';

my $nargs = $#ARGV + 1;
if($nargs != 4) {
    print "SetAnalysisLevel xmlin xmlout level name\n";
    exit(1);
}

my $xml_in   = shift @ARGV;
my $xml_out  = shift @ARGV;
my $analevel = shift @ARGV;
my $ananame  = shift @ARGV;

if(! -e $xml_in ) {
  print "ERROR: input xml file $xml_in does not exist.\n";
  exit(1);
}


#############################################################################
# Standard Stuff to read in an XML document
#############################################################################
my $parser = new XML::DOM::Parser;
my $doc = $parser->parsefile("$xml_in");
my $root_node = $doc->getDocumentElement();

# Change the name and level of first (and should be only) series
setValue($doc,"FIPS//series[0]//analysislevel//level",$analevel);
setValue($doc,"FIPS//series[0]//analysislevel//name",$ananame);
# Change the run number if this is higher than first-level analysis
if(($analevel !~ FirstLevel) && ($analevel !~ PreProc)) {
  setValue($doc,"FIPS//series[0]//number",0);
}
$doc->printToFile($xml_out);

$doc->dispose; # Avoid memory leaks - cleanup circular references

exit(0);
