# FIPS-XMLFuncs.pl
# $Id: FIPS-XMLFuncs.pl,v 1.1 2006/05/31 18:59:12 rnotestine Exp $
################################################################################
# Dependencies:
#		XML::XPath - SRB@/home/dkeator@uci-bic/NonHumanPhantomScript/XML
#
# Authors: 	David Keator (dbkeator@uci.edu)
#		Syam Gadde (gadde@biac.duke.edu)
#               Doug Greve (greve@nmr.mgh.harvard.edu)
#		
# Revision History: Modification of LocalHumanUpload/XMLFuncs.pl 
################################################################################
use XML::DOM;
use XML::XQL;
use XML::XQL::DOM;

our $ELEMENT_NODE = 1;
our $TEXT_NODE = 3;

################################################################################
# traverse
# Prints out an XML tree starting at the node you pass it
################################################################################
sub traverse {
	my($node)=@_;
	#print "node_type = ", $node->getNodeType,"\n";

	if($node->getNodeType == $ELEMENT_NODE) {
		print "<", $node->getNodeName, ">";
		foreach my $child ($node->getChildNodes()) {
			traverse($child);
		}
		print "</", $node->getNodeName, ">";
	} elsif ($node->getNodeType() == $TEXT_NODE) {
		print $node->getData;
	} 
}

################################################################################
# getTextContents
# Works with getValue function to get the text contents of a node
################################################################################
sub getTextContents {
  my ($node, $strip)= @_;
  my $contents;

  if (! $node ) 
  { 
    return; 
  }
  for my $child ($node->getChildNodes()) {
    if ( !($child->getNodeType == $ELEMENT_NODE)  ) {
       $contents .= $child->getData();
    }
  }

  if ($strip) {
    $contents =~ s/^\s+//;
    $contents =~ s/\s+$//;
  }

  return $contents;
}

################################################################################
# findNode
# Returns the first matching node for a specified document pointer and XPath
################################################################################
sub findNode {
  my ($node, $xpath) = @_;
  if (! defined($node) || ! defined($xpath) )
  {
    return undef;
  }
  my $match = ($node->xql($xpath))[0];
  if (! $match )
  {
    return undef;
  }
  return $match;
}

################################################################################
# getValue
# Gets the value of a node when you pass it the document pointer of an XPath to
# the node you're interested in
################################################################################
sub getValue {
  my ($node, $xpath) = @_;
  my $match = findNode( $node, $xpath );
  if (! defined($match) )
  {
    print "node can't be found!";
    return undef;
  }
  return getTextContents( $match );
}

################################################################################
# setValue
# Sets the value of a node, use it by passing the document pointer, an XPath to
# the node, and a new value
################################################################################
sub setValue {
  my ($node, $xpath, $value) = @_;
  my $match = findNode( $node, $xpath );
  if (! defined($match) )
  {
    return undef;
  }
  
  foreach my $child ( $match->getChildNodes() ) 
  {
    $match->removeChild($child);
  }
  $match->addText($value);
  return $match;
}

################################################################################
# appendNodeXPath
# Appends a node, "nodename" to the document as a child of location "xpath" giving it "value"
################################################################################
sub appendNodeXPath {
  my ($doc, $nodename, $xpath, $value) = @_;

  if (! defined($nodename) || ($nodename eq "") ) {
    return undef;
  }

  my $match = findNode( $doc, $xpath );
  if (! defined($match) )
  {
    return undef;
  }

  my $newnode;
  eval {
    $newnode = $doc->createElement( $nodename );
  };

  if ($@ || (! defined($newnode) )) {
    return undef;
  }
  
  $match->appendChild( $newnode );
  
  if ( defined($value) ) {
    $newnode->addText($value);
  }

  return $newnode;
}

################################################################################
# appendNode
# Appends a node, "nodename" to the document as a child of "node" giving it "value"
################################################################################
sub appendNode {
  my ($doc, $node, $nodename, $value) = @_;

  if (! defined($nodename) || ($nodename eq "") || !defined($node) ) {
    return undef;
  }

  my $newnode;
 eval {
    $newnode = $doc->createElement( $nodename );
  };
 
  if ((! defined($newnode) )) {
    print "not defined";
    return undef;
  }
  
  $node->appendChild( $newnode );
  
  if ( defined($value) ) {
    $newnode->addText($value);
  }

  return $newnode;
}

################################################################################
# removeChild
# removes child node corresponding to XPATH....
################################################################################
sub removeChild {
  my ($node, $xpath) = @_;
  
  if (! defined($node) || ! defined($xpath) )
  {
    print "upps";
    return undef;
  }
  my $match = findNode($node,$xpath);
  if (! $match )
  {
    print "no match";
    return undef;
  } else {
  	
  	my $parent = $match->getParentNode();
  	$parent->removeChild($match);
  	
  	return
  }	
	
}

################################################################################
# copySubTree
# Copies the sourcenode as a child of the destnode
################################################################################
sub copySubTree
{
  my ($sourcenode, $destnode) = @_;

  my $copy_node =  $sourcenode->cloneNode(1);
  if ( $sourcenode->getOwnerDocument() ne $destnode->getOwnerDocument() ) 
  {
    $copy_node->setOwnerDocument( $destnode->getOwnerDocument() );
  }
  $destnode->appendChild($copy_node);
  return $copy_node;
}
