#!/usr/bin/perl 
#
# GetSliceOrder.pl - extracts the slice order from fips-process.xml
# Assumes that it is in /FIPS/series/sliceorder and that they 
# are numbers separated by commas. If there are multiple series,
# then the first one is used. 
#
# Prints the slice order to the terminal as numbers separated by
# spaces and exits with 0 status. If there is an error, the
# error is printed to the terminal, and the exit status is 1.
#
# The slice indices start at 1. The slice order indicates the order
# in which the anatomical slices were acquired. Ie, 1 3 5 2 4 6 
# would indicate that the 3rd slice in the volume stored on disk
# was acquired 2nd.
#
# $Id: GetSliceOrder.pl,v 1.2 2006/01/26 22:02:52 dgreve Exp $

use strict;
use FindBin qw($RealDir);
use lib "$RealDir";
use XML::XPath;
use XML::XPath::XMLParser;
use XML::Simple;

# main --------------------------------------------------------#
#--------------------------------------------------------------#
#--------------------------------------------------------------#

my $xmlfile = ();

if($#ARGV == -1) {
    print "GetSliceOrder.pl\n" ;
    print "  --xml  fips-process.xml\n" ;
    print "  \n" ;
    exit 1;
}

# Parse the command-line
while($#ARGV != -1){
    my $inarg = shift @ARGV;
    #print "inarg: $inarg, $#ARGV\n";
    if($inarg eq "--xml") {
	if($#ARGV == -1){
	    print "ERROR: flag $inarg needs one argument\n";
	    exit 1;
	}
	$xmlfile = shift @ARGV;
    } else {
	print "ERROR: flag $inarg unrecognized\n";
	exit 1;
    }
}

# Check the arguments
if($xmlfile eq "") {
    print "ERROR: need xml file\n";
    exit 1;
}
if( ! -e $xmlfile ){
    print "ERROR: $xmlfile does not exist\n";
    exit 1;
}

my $sliceorder = ();
my $d = XMLin($xmlfile, keeproot=>1);
my $series = $d->{FIPS}->{series};
my $varref = ref($series);
if( $varref eq "HASH") {
  # There is only one series
  $sliceorder = $d->{FIPS}->{series}->{sliceorder};
} else {
  # There are multiple series, use the first
  $sliceorder = $d->{FIPS}->{series}->[0]->{sliceorder};
}
my @darray = split(",",$sliceorder);

if( $#darray == -1 ) {
    print "ERROR: parsing $xmlfile\n";
    exit 1;
}

print "@darray\n";

exit 0;

#~/links/srb-home/fBIRNPhaseII__0010/000600042105/Visit__0006__L0001/bay4-trio__L0001/bh1/Native/Original__0001/NIFTI_1.0/f.xml
