#!/usr/bin/perl 
#
# GetVisitDate.pl - extracts the visit date from fips-process.xml
# Assumes that it is in /FIPS/visit/visitDate
#
# Prints the visit date to the terminal.
# If there is an error, the
# error is printed to the terminal, and the exit status is 1.
#
#
# $Id: GetVisitDate.pl,v 1.2 2007/10/10 20:59:16 dgreve Exp $

use strict;
use FindBin qw($RealDir);
use lib "$RealDir";
use XML::XPath;
use XML::XPath::XMLParser;
use XML::Simple;

# main --------------------------------------------------------#
#--------------------------------------------------------------#
#--------------------------------------------------------------#

my $xmlfile = ();

if($#ARGV == -1) {
    print "GetVisitDate.pl\n" ;
    print "  --xml  fips-process.xml\n" ;
    print "  \n" ;
    exit 1;
}

# Parse the command-line
while($#ARGV != -1){
    my $inarg = shift @ARGV;
    #print "inarg: $inarg, $#ARGV\n";
    if($inarg eq "--xml") {
	if($#ARGV == -1){
	    print "ERROR: flag $inarg needs one argument\n";
	    exit 1;
	}
	$xmlfile = shift @ARGV;
    } else {
	print "ERROR: flag $inarg unrecognized\n";
	exit 1;
    }
}

# Check the arguments
if($xmlfile eq "") {
    print "ERROR: need xml file\n";
    exit 1;
}
if( ! -e $xmlfile ){
    print "ERROR: $xmlfile does not exist\n";
    exit 1;
}

my $d = XMLin($xmlfile, keeproot=>1);
my $visitdate = $d->{FIPS}->{visit}->{visitDate};

if( $visitdate eq "" ) {
    #print "ERROR: parsing $xmlfile\n";
    #exit 1;
    $visitdate = "unknown";
}

print "$visitdate\n";

exit 0;


