#!/usr/bin/perl 
# MkFlirtInitMtx.pl

use strict;
use FindBin qw($RealDir);
use lib "$RealDir";
use XML::XPath;
use XML::XPath::XMLParser;
use XML::Simple;

#--------------------------------------------------------------#
sub det4x4 {
    my ($r11,$r12,$r13,$v1,$r21,$r22,$r23,$v2,$r31,$r32,$r33,$v3) = @_;

    my $deti =
	($r11*$r22*$r33)-($r11*$r32*$r23)-($r21*$r12*$r33)+
	($r21*$r32*$r13)+($r31*$r12*$r23)-($r31*$r22*$r13);
    return $deti;
}

#--------------------------------------------------------------#
sub print4x4{
    my @argin = @_;
    my @mtx = @argin[0..15];
    my $fname = "";
    my $i = 0;
    my $r = 0;
    my $c = 0;

    if($#argin == 16) {
	$fname = @argin[16];
	open(fp,">$fname");
    }
    for($r = 0 ; $r <= 3; $r++) {
	for($c = 0 ; $c <= 3; $c++) {
	    if($#argin == 16) {print fp @mtx[$i]," ";}
	    else {print @mtx[$i]," ";}
	    $i++;
	}
	if($#argin == 16) {print fp "\n";
	} else {print "\n";}
    }
    if($#argin == 16) {	close(fp);}
    return;
}

#--------------------------------------------------------------#
sub vox2ras_from_xml {
    my $xmlfile = @_[0];
    #print "xmlfile $xmlfile\n";
    my $d = XMLin($xmlfile, keeproot=>1);

    my @ras0  = split(" ",$d->{serieslevel}->{datarec}->{rasorigin});
    my $xtype = $d->{serieslevel}->{datarec}->{dimension}->[0]->{type};
    my $xsize = $d->{serieslevel}->{datarec}->{dimension}->[0]->{spacing};
    my @xdc   = split(" ",$d->{serieslevel}->{datarec}->{dimension}->[0]->{direction});
    
    my $ytype = $d->{serieslevel}->{datarec}->{dimension}->[1]->{type};
    my $ysize = $d->{serieslevel}->{datarec}->{dimension}->[1]->{spacing};
    my @ydc   = split(" ",$d->{serieslevel}->{datarec}->{dimension}->[1]->{direction});
    
    my $ztype = $d->{serieslevel}->{datarec}->{dimension}->[2]->{type};
    my $zsize = $d->{serieslevel}->{datarec}->{dimension}->[2]->{spacing};
    my @zdc   = split(" ",$d->{serieslevel}->{datarec}->{dimension}->[2]->{direction});
    
    my $m11 = $xdc[0] * $xsize;
    my $m21 = $xdc[1] * $xsize;
    my $m31 = $xdc[2] * $xsize;
    my $m41 = 0;
    
    my $m12 = $ydc[0] * $ysize;
    my $m22 = $ydc[1] * $ysize;
    my $m32 = $ydc[2] * $ysize;
    my $m42 = 0;
    
    my $m13 = $zdc[0] * $zsize;
    my $m23 = $zdc[1] * $zsize;
    my $m33 = $zdc[2] * $zsize;
    my $m43 = 0;
    
    my $m14 = @ras0[0];
    my $m24 = @ras0[1];
    my $m34 = @ras0[2];
    my $m44 = 1;
    
    return($m11,$m12,$m13,$m14,
	   $m21,$m22,$m23,$m24,
	   $m31,$m32,$m33,$m34,
	   $m41,$m42,$m43,$m44);
}

#--------------------------------------------------------------#
sub PrintUsage {
    print "\n";
    print "IsRightHandedXML.pl\n" ;
    print "  --xml  involxml\n" ;
    print "  --help\n" ;
    print "  \n" ;
}

#--------------------------------------------------------------#
sub PrintHelp {
    PrintUsage;
    print "Determines whether voxel coordinate system is right-handed.\n";
    print "If so, a 1 is printed to stdout. Otherwise a 0 is printed.\n";
    print "Any error causes the exit status to be non-zero (otherwise 0).\n";
    print "\n";
    print " --xml involxml\n";
    print "\n";
    print "   XML/BXH file of the input volume\n";
    print "\n";
}

#--------------------------------------------------------------#
#--------------------------------------------------------------#
#--------------------------------------------------------------#
my $involxml = ();

if($#ARGV == -1) {
    PrintUsage;
    exit 1;
}

# Parse the command-line
while($#ARGV != -1){
    my $inarg = shift @ARGV;
    #print "inarg: $inarg, $#ARGV\n";
    if($inarg eq "--xml") {
	if($#ARGV == -1){
	    print "ERROR: flag $inarg needs one argument\n";
	    exit 1;
	}
	$involxml = shift @ARGV;
    } elsif($inarg eq "--help") {
	PrintHelp;
	exit 1;
    } else {
	print "ERROR: flag $inarg unrecognized\n";
	exit 1;
    }
}

# Check the arguments
if($involxml eq "") {
    print "ERROR: no input xml specified\n";
    exit 1;
}
if(! -e $involxml) {
   print "ERROR: Input xml file $involxml does not exist\n";
    exit 1;
}

# Read in the input vox2ras
my @vox2ras_in = vox2ras_from_xml($involxml);
#print "Input Vox2RAS-----------------\n";
#print4x4(@vox2ras_in);

my $vox2ras_in_det = det4x4(@vox2ras_in);

#print "det: ", $vox2ras_in_det,"\n";
if($vox2ras_in_det > 0) {print "1\n";} # Right-Handed
if($vox2ras_in_det < 0) {print "0\n";} # Left-Handed

exit 0;

#~/links/srb-home/fBIRNPhaseII__0010/000600042105/Visit__0006__L0001/bay4-trio__L0001/bh1/Native/Original__0001/NIFTI_1.0/f.xml
