#!/usr/bin/perl 
# MkFlirtInitMtx.pl

use strict;
use FindBin qw($RealDir);
use lib "$RealDir";
use XML::XPath;
use XML::XPath::XMLParser;
use XML::Simple;

#--------------------------------------------------------------#
sub mtxdeterminant {
    my ($r11,$r12,$r13,$v1,$r21,$r22,$r23,$v2,$r31,$r32,$r33,$v3) = @_;

    my $det =
	($r11*$r22*$r33)-($r11*$r32*$r23)-($r21*$r12*$r33)+
	($r21*$r32*$r13)+($r31*$r12*$r23)-($r31*$r22*$r13);
    return($det);
}

#--------------------------------------------------------------#
sub inv4x4 {
    my ($r11,$r12,$r13,$v1,$r21,$r22,$r23,$v2,$r31,$r32,$r33,$v3) = @_;

    my $deti = mtxdeterminant(@_);
    if ($deti != 0.0) {
	$deti = 1.0 / $deti;
    }
    
    my ($q11,$q12,$q13,$w1,$q21,$q22,$q23,$w2,$q31,$q32,$q33,$w3);
    $q11 = $deti*( $r22*$r33-$r32*$r23) ;
    $q12 = $deti*(-$r12*$r33+$r32*$r13) ;
    $q13 = $deti*( $r12*$r23-$r22*$r13) ;
    $w1  = $deti*((-$r12*$r23*$v3)
		  +($r12*$v2*$r33)
		  +($r22*$r13*$v3)
		  -($r22*$v1*$r33)
		  -($r32*$r13*$v2)
		  +($r32*$v1*$r23));
    
    $q21 = $deti*(-$r21*$r33+$r31*$r23) ;
    $q22 = $deti*( $r11*$r33-$r31*$r13) ;
    $q23 = $deti*(-$r11*$r23+$r21*$r13) ;
    $w2  = $deti*(($r11*$r23*$v3)
		  -($r11*$v2*$r33)
		  -($r21*$r13*$v3)
		  +($r21*$v1*$r33)
		  +($r31*$r13*$v2)
		  -($r31*$v1*$r23));

    $q31 = $deti*( $r21*$r32-$r31*$r22) ;
    $q32 = $deti*(-$r11*$r32+$r31*$r12) ;
    $q33 = $deti*( $r11*$r22-$r21*$r12) ;
    $w3  = $deti*((-$r11*$r22*$v3)
		  +($r11*$r32*$v2)
		  +($r21*$r12*$v3)
		  -($r21*$r32*$v1)
		  -($r31*$r12*$v2)
		  +($r31*$r22*$v1));


    my ($q41,$q42,$q43,$w4) = (0, 0, 0, 1);
    if ($deti == 0) {
	$w4 = 0;
    }

    return($q11,$q12,$q13,$w1,
	   $q21,$q22,$q23,$w2,
	   $q31,$q32,$q33,$w3,
	   $q41,$q42,$q43,$w4);

}

#--------------------------------------------------------------#
sub mult4x4{
    my @mtx1 = @_[ 0..15];
    my @mtx2 = @_[16..31];

    my $i1 = 0;
    my $c1 = 0;
    my $r1 = 0;
    my $c2 = 0;
    my $i2 = 0;
    my $r2 = 0;
    my $c2 = 0;
    my $i3 = 0;
    my @mtx;

    for ($r1 = 0; $r1 <= 3; $r1++){
	for ($c2 = 0; $c2 <= 3; $c2++){
	    @mtx[$i3] = 0;
	    for ($c1 = 0; $c1 <= 3; $c1++){
		$i1 = $c1 + 4*$r1;
		$r2 = $c1;
		$i2 = $c2 + 4*$r2;
		@mtx[$i3] += $mtx1[$i1]*$mtx2[$i2];
	    }
	    $i3 ++;
	}
    }
    return @mtx;
}
#--------------------------------------------------------------#
sub print4x4{
    my @argin = @_;
    my @mtx = @argin[0..15];
    my $fname = "";
    my $i = 0;
    my $r = 0;
    my $c = 0;

    if($#argin == 16) {
	$fname = @argin[16];
	open(fp,">$fname");
    }
    for($r = 0 ; $r <= 3; $r++) {
	for($c = 0 ; $c <= 3; $c++) {
	    if($#argin == 16) {print fp @mtx[$i]," ";}
	    else {print @mtx[$i]," ";}
	    $i++;
	}
	if($#argin == 16) {print fp "\n";
	} else {print "\n";}
    }
    if($#argin == 16) {	close(fp);}
    return;
}

#--------------------------------------------------------------#
sub find_datarec {
  my ($d,) = @_;
  my $datarec = undef;
  my @queue = ($d);
  while (scalar(@queue) > 0) {
    my $curnode = shift @queue;
    if (exists $curnode->{datarec} && exists $curnode->{datarec}->{rasorigin}) {
      $datarec = $curnode->{datarec};
      last;
    }
    for my $childname (keys %{$curnode}) {
      if (ref($curnode->{$childname}) eq 'HASH') {
	push @queue, $curnode->{$childname};
      }
    }
  }
  return $datarec;
};

#--------------------------------------------------------------#
sub vox2ras_from_xml {
    my $xmlfile = @_[0];
    #print "xmlfile $xmlfile\n";
    my $d = XMLin($xmlfile, keeproot=>1);
    my $datarec = find_datarec($d);

    my @ras0  = split(" ",$datarec->{rasorigin});
    my $xtype = $datarec->{dimension}->[0]->{type};
    my $xsize = $datarec->{dimension}->[0]->{spacing};
    my @xdc   = split(" ",$datarec->{dimension}->[0]->{direction});
    
    my $ytype = $datarec->{dimension}->[1]->{type};
    my $ysize = $datarec->{dimension}->[1]->{spacing};
    my @ydc   = split(" ",$datarec->{dimension}->[1]->{direction});
    
    my $ztype = $datarec->{dimension}->[2]->{type};
    my $zsize = $datarec->{dimension}->[2]->{spacing};
    my @zdc   = split(" ",$datarec->{dimension}->[2]->{direction});
    
    my $m11 = $xdc[0] * $xsize;
    my $m21 = $xdc[1] * $xsize;
    my $m31 = $xdc[2] * $xsize;
    my $m41 = 0;
    
    my $m12 = $ydc[0] * $ysize;
    my $m22 = $ydc[1] * $ysize;
    my $m32 = $ydc[2] * $ysize;
    my $m42 = 0;
    
    my $m13 = $zdc[0] * $zsize;
    my $m23 = $zdc[1] * $zsize;
    my $m33 = $zdc[2] * $zsize;
    my $m43 = 0;
    
    my $m14 = @ras0[0];
    my $m24 = @ras0[1];
    my $m34 = @ras0[2];
    my $m44 = 1;
    
    return($m11,$m12,$m13,$m14,
	   $m21,$m22,$m23,$m24,
	   $m31,$m32,$m33,$m34,
	   $m41,$m42,$m43,$m44);
}
#--------------------------------------------------------------#
sub fsl_vox2ras_from_xml {
    my $xmlfile = @_[0];
    my $d = XMLin($xmlfile, keeproot=>1);
    my $datarec = find_datarec($d);

    my $ysize = $datarec->{dimension}->[1]->{spacing};
    my $zsize = $datarec->{dimension}->[2]->{spacing};

    my @xdc   = split(" ",$datarec->{dimension}->[0]->{direction});
    my @ydc   = split(" ",$datarec->{dimension}->[1]->{direction});
    my @zdc   = split(" ",$datarec->{dimension}->[2]->{direction});
    my @ras0  = split(" ",$datarec->{rasorigin});

    # Computing the first row is a little tricky. Here are notes from mri.c
    # -------------------------------------------------------------------
    # This next section of code alters the way the reg mat is computed
    # based on the current FSL format and the determinant of the vox2ras
    # matrices. When using ANALYZE, FSL did not know what the true
    # vox2ras was. However, with NIFTI, FSL does know, and it treats the
    # data differently depending upon whether the vox2ras has a negative
    # or positive determinant. If positive, FSL (flirt) will flip the
    # data left-right internally to make it negative, and this has to be
    # taken into account when computing the tkreg matrix. It is not
    # possible to stop FSL/flirt from flipping the data, even when
    # supplying an initial matrix that has a flip in it.  With ANALYZE,
    # FSL always assumes negative determinant, so no flipping  occurs.
    # This change was also implemented in MRItkreg2FSL().
    # -------------------------------------------------------------------*/
    my $xsize;
    my $xoff;
    my $det = mtxdeterminant(@xdc, $ras0[0], @ydc, $ras0[1], @zdc, $ras0[2], 0, 0, 0, 1);
    my $fslouttype = "$ENV{FSLOUTPUTTYPE}";
    if($fslouttype eq "ANALYZE" || $det < 0){
	$xsize = $datarec->{dimension}->[0]->{spacing};
	$xoff  = 0;
    }
    if(!($fslouttype eq "ANALYZE") && $det > 0){
	printf "Positive determinant and non-analyze format\n";
	$xsize = -$datarec->{dimension}->[0]->{spacing};
	$xoff  = abs($xsize)*($datarec->{dimension}->[0]->{size}-1);
    }

    return($xsize,0,0,$xoff,
	   0,$ysize,0,0,
	   0,0,$zsize,0,
	   0,0,0,1);
}
#--------------------------------------------------------------#
sub vox2ras_stdref {

    return(-2,0,0,  91,
	    0,2,0,-109,
	    0,0,2, -91,
	    0,0,0,   1);
}

#--------------------------------------------------------------#
sub fsl_vox2ras_stdref {

    return( 2,0,0,0,
	    0,2,0,0,
	    0,0,2,0,
	    0,0,0,1);
}

#--------------------------------------------------------------#
#--------------------------------------------------------------#
#--------------------------------------------------------------#
my $involxml = ();
my $refvolxml = ();
my $usestdref = 0;
my $outmatfile = ();

if($#ARGV == -1) {
    print "MkFlirtInitMtx.pl\n" ;
    print "  --in  invol.xml\n" ;
    print "  --ref refvol.xml\n" ;
    print "  --stdref : use FSL Std Brain vox2ras\n" ;
    print "  --o outmatfile : use as init to Flirt\n" ;
    print "  \n" ;
    exit 1;
}

# Parse the command-line
while($#ARGV != -1){
    my $inarg = shift @ARGV;
    #print "inarg: $inarg, $#ARGV\n";
    if($inarg eq "--in") {
	if($#ARGV == -1){
	    print "ERROR: flag $inarg needs one argument\n";
	    exit 1;
	}
	$involxml = shift @ARGV;
    } elsif($inarg eq "--ref") {
	if($#ARGV == -1){
	    print "ERROR: flag $inarg needs one argument\n";
	    exit 1;
	}
	$refvolxml = shift @ARGV;
    } elsif($inarg eq "--stdref") {
	$usestdref = 1;
    } elsif($inarg eq "--o") {
	if($#ARGV == -1){
	    print "ERROR: flag $inarg needs one argument\n";
	    exit 1;
	}
	$outmatfile = shift @ARGV;
    } else {
	print "ERROR: flag $inarg unrecognized\n";
	exit 1;
    }
}

# Check the arguments
if($involxml eq "") {
    print "ERROR: no input volume specified\n";
    exit 1;
}
if($refvolxml eq "" && ! $usestdref) {
    print "ERROR: no ref volume specified\n";
    exit 1;
}
if(!($refvolxml eq "") && $usestdref) {
    print "ERROR: cannot specify both ref and stdref\n";
    exit 1;
}
if(! -e $involxml) {
   print "ERROR: Input volume $involxml does not exist\n";
    exit 1;
}
if( ! ($refvolxml eq "") && (! -e $refvolxml) ){
    print "ERROR: ref volume $refvolxml does not exist\n";
    exit 1;
}
if($outmatfile eq "") {
    print "ERROR: no output matrix file specified\n";
    exit 1;
}
# Make sure environment is set up correctly
if(! $ENV{FSLOUTPUTTYPE} ){
    print("ERROR: cannot find environment variable FSLOUTPUTTYPE.\n");
    exit(1);
}
my $fslouttype = "$ENV{FSLOUTPUTTYPE}";
print("FSLOUTPUTTYPE $fslouttype \n");

# Read in the input vox2ras
my @vox2ras_in = vox2ras_from_xml($involxml);
print "Input Vox2RAS-----------------\n";
print4x4(@vox2ras_in);
my @inv_vox2ras_in = inv4x4(@vox2ras_in);

my @fsl_vox2ras_in = fsl_vox2ras_from_xml($involxml);
print "Input FSL Vox2RAS-----------------\n";
print4x4(@fsl_vox2ras_in);
my @inv_fsl_vox2ras_in = inv4x4(@fsl_vox2ras_in);

# Read in the reference vox2ras
my @vox2ras_ref;
my @fsl_vox2ras_ref; 
if($usestdref) {
    @vox2ras_ref = vox2ras_stdref;
    @fsl_vox2ras_ref = fsl_vox2ras_stdref;
} else {
    @vox2ras_ref = vox2ras_from_xml($refvolxml);
    @fsl_vox2ras_ref = fsl_vox2ras_from_xml($refvolxml);
}
my @inv_vox2ras_ref = inv4x4(@vox2ras_ref);
my @inv_fsl_vox2ras_ref = inv4x4(@fsl_vox2ras_ref);

print "Ref Vox2RAS -----------------\n";
print4x4(@vox2ras_ref);

print "Ref FSL Vox2RAS -----------------\n";
print4x4(@fsl_vox2ras_ref);

print "-----------------\n";
# pm =   v2rreffsl*inv(v2rref)*v2rin*inv(v2rinfsl)
my @pm;
@pm = mult4x4(@vox2ras_in,@inv_fsl_vox2ras_in);
@pm = mult4x4(@inv_vox2ras_ref,@pm);
@pm = mult4x4(@fsl_vox2ras_ref,@pm);

print4x4(@pm);

print4x4(@pm,$outmatfile);



exit 0;

#~/links/srb-home/fBIRNPhaseII__0010/000600042105/Visit__0006__L0001/bay4-trio__L0001/bh1/Native/Original__0001/NIFTI_1.0/f.xml
