% fips_qual_summ_avg(summfile)
%   Compute averages of data quality summary metrics
%   for each site and each subject
%   summfile:	Name of input text file containing data quality
%		summary metrics, as produced by 'fips-qual-summ'

function fips_qual_summ_avg(summfile)

[birnid, recsite, acqsite, proj, studyname, studyid, visitname, visitid, ...
 runtype, paradigm, runno, analevel, ananame, fwhm, res, mabs, mrel] ...
 = textread(summfile, '%s%s%s%s%s%s%s%s%s%s%s%s%s%n%n%n%n', 'commentstyle', 'matlab');

idot = strfind(summfile, '.');
if isempty(idot)
	avgfile = [summfile, '.avg'];
else
	idot = idot(end);
	avgfile = [summfile(1:idot-1), '.avg', summfile(idot:end)];
end
avgfid = fopen(avgfile, 'w');
fprintf(avgfid, ...
 'Site\tSubject\tFWHM(mm)\t\tAvgNormResidual\t\tAbsMeanMotion(mm)\t\tRelMeanMotion(mm)\n');
fprintf(avgfid, '\t\tAvg\tStd\tAvg\tStd\tAvg\tStd\tAvg\tStd\n');

site = unique(acqsite);

for is = 1:length(site)

	ii = strcmp(acqsite, site(is));

	fwhm_avg_site(is) = mean(fwhm(ii));
	fwhm_std_site(is) = std(fwhm(ii));

	res_avg_site(is) = mean(res(ii));
	res_std_site(is) = std(res(ii));

	mabs_avg_site(is) = mean(mabs(ii));
	mabs_std_site(is) = std(mabs(ii));

	mrel_avg_site(is) = mean(mrel(ii));
	mrel_std_site(is) = std(mrel(ii));

	fprintf(avgfid, ...
	 '%s\tAll\t%1.3f\t%1.3f\t%1.3f\t%1.3f\t%1.3f\t%1.3f\t%1.3f\t%1.3f\n', ...
	 cell2mat(site(is)), ...
	 fwhm_avg_site(is), fwhm_std_site(is), ...
	 res_avg_site(is), res_std_site(is), ...
	 mabs_avg_site(is), mabs_std_site(is), ...
	 mrel_avg_site(is), mrel_std_site(is));
end

subj = unique(birnid);

for is = 1:length(subj)
	ii = strcmp(birnid, subj(is));

	fwhm_avg_subj(is) = mean(fwhm(ii));
	fwhm_std_subj(is) = std(fwhm(ii));

	res_avg_subj(is) = mean(res(ii));
	res_std_subj(is) = std(res(ii));

	mabs_avg_subj(is) = mean(mabs(ii));
	mabs_std_subj(is) = std(mabs(ii));

	mrel_avg_subj(is) = mean(mrel(ii));
	mrel_std_subj(is) = std(mrel(ii));

	fprintf(avgfid, ...
	 'All\t%s\t%1.3f\t%1.3f\t%1.3f\t%1.3f\t%1.3f\t%1.3f\t%1.3f\t%1.3f\n', ...
	 cell2mat(subj(is)), ...
	 fwhm_avg_subj(is), fwhm_std_subj(is), ...
	 res_avg_subj(is), res_std_subj(is), ...
	 mabs_avg_subj(is), mabs_std_subj(is), ...
	 mrel_avg_subj(is), mrel_std_subj(is));
end

fclose(avgfid);

