#
# $Id: fips-env.csh,v 1.2 2006/11/15 23:53:08 rnotestine Exp $
#
# Sets up environment variables. This is run at the begining of 
# each FIPS script.
#
# User may define LOCAL_SRB_HOME_BIRN (where pixel data go) 
#   If not defined, then sources ~/.fipsrc
#     If ~/.fipsrc does not exist, then error (exit 1)
#     If still not defined, then error (exit 1)
#
# Note: when a variable "must" be defined, it can be defined
# in your ~/.fipsrc file as well as any other place in your
# environment. Your ~/.fipsrc file will override your current
# environment.
#
# User must define FIPS_HOME (where FLACs and Database go)
#
# User may define FIPS_FLAC_DIR
#  If not defined, then setenv FIPS_FLAC_DIR $FIPS_HOME
#
# User may define FIPS_DB_DIR
#  If not defined, then setenv FIPS_DB_DIR $FIPS_HOME/db
#
# User may define $FIPS_GLOBAL_SCHEDULES_DIR
#  If not defined, then 
#    setenv FIPS_GLOBAL_SCHEDULES_DIR $FIPSDIR/stimulus-schedules
#
# This script also sets the following variables:
#    setenv FIPS_OUTPUT_TYPE NIFTI_GZ
#    setenv FSLOUTPUTTYPE $FIPS_OUTPUT_TYPE
#    setenv MY_FIPS_DIR $FIPS_DB_DIR  # for backwards compate
#    setenv FIPSETCDIR = $FIPSDIR/etc
#
# It also does some FSL setup
#   source $FSLDIR/etc/fslconf/fsl.csh
#   setenv FSLOUTPUTTYPE NIFTI_GZ
#

if(! $?FIPSDIR) then
  setenv FIPSBINDIR  `dirname $0`;
  setenv FIPSDIR     `dirname $FIPSBINDIR`;
else
  setenv FIPSBINDIR  $FIPSDIR/bin
endif
setenv FIPSETCDIR $FIPSDIR/etc

# Source the FIPS RC File if needed
set fipsrc = ~/.fipsrc
if( -e $fipsrc) source $fipsrc

if($?FIPS_ROOT) setenv LOCAL_SRB_HOME_BIRN $FIPS_ROOT
if(! $?LOCAL_SRB_HOME_BIRN) then
  echo "ERROR: variable LOCAL_SRB_HOME_BIRN not defined"
  exit 1;
endif

if(! $?FIPS_HOME) then
  echo "ERROR: variable FIPS_HOME not defined"
  exit 1;
endif

if(! $?FIPS_FLAC_DIR) then
  setenv FIPS_FLAC_DIR $FIPS_HOME
endif
if(! $?FIPS_DB_DIR) then
  setenv FIPS_DB_DIR $FIPS_HOME/db
endif

setenv MY_FIPS_DIR $FIPS_DB_DIR # for backwards compat

# Place for global stimulus schedules to go
if(! $?FIPS_GLOBAL_SCHEDULES_DIR) then
  setenv FIPS_GLOBAL_SCHEDULES_DIR $FIPSDIR/stimulus-schedules
endif

# Setup the output format (dont change this)
if(! $?FIPS_OUTPUT_TYPE) then
  setenv FIPS_OUTPUT_TYPE NIFTI_GZ
endif

# Source FSL config and force FSLOUTPUTTYPE to match FIPS_OUTPUT_TYPE
if(! $?FSLDIR) then
  echo "ERROR: variable FSLDIR not defined"
  exit 1;
endif
if(! -e $FSLDIR) then
  echo "ERROR: cannot find $FSLDIR"
  exit 1;
endif

source $FSLDIR/etc/fslconf/fsl.csh
setenv FSLOUTPUTTYPE $FIPS_OUTPUT_TYPE


exit 0
