#!/usr/bin/python
"""
FBIRN Image Processing Stream (FIPS) routines.
$Id: fips.py,v 1.35 2008/11/19 00:39:23 dgreve Exp $
"""
import sys;
import os;
import os.path;
from os.path import join;
import xml.dom;
from xml.dom.ext.reader import Sax2
from xml.dom.ext import PrettyPrint;
import time;
import datetime;
import string;
import fips; # Needed when calling funcs with fips.     
import pdb; # pdb.set_trace();
import statvfs
from copy import deepcopy;
import tempfile;

UseSQLite = os.environ.get("FIPS_USE_SQLITE",0);
if(UseSQLite == "1"):
  from pysqlite2 import dbapi2 as sqlite;
#endif


# mkdir someplace/fips
# setenv PYTHONPATH somplace
# import fips

# DB Entries:
# 0 nth
# 1 bid
# 2 rec site
# 3 acq site
# 4 proj
# 5 studyname
# 6 studyid
# 7 visitname
# 8 visitid
# 9 visitdate
# 10 seriestype
# 11 paradigm
# 12 seriesnum
# 13 analevel
# 14 ananame
# 15 anadate
# 16 anatime
# 17 errorstat
# 18 isrunning
# 19 xmldir

#---------------------------------------------------------
def mkdirp(path):
  """
  err = mkdirp(path);
  Like unix mkdir -p path
  """
  if(os.path.exists(path)): return 0;
  try:
    os.makedirs(path);
    return 0;
  except:
    print "ERROR: making %s" % path;
    return 1;
  #end try
#def mkdirp(path):

#---------------------------------------------------------
def SetBIRNID(fpdoc,birnid):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  bidnodes = fipsnode.getElementsByTagName("BIRNID");
  fipsnode.removeChild(bidnodes[0]);
  bidnode = fpdoc.createElement("BIRNID");  
  tx = fpdoc.createTextNode(birnid);
  bidnode.appendChild(tx);
  fipsnode.appendChild(bidnode);
  return fpdoc;
# SetBIRNID();

#---------------------------------------------------------
def SetAcqSiteID(fpdoc,AcqSiteID):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  nodes = fipsnode.getElementsByTagName("acquisitionSiteID");
  if(len(nodes) > 0):
    fipsnode.removeChild(nodes[0]);
  node = fpdoc.createElement("acquisitionSiteID");
  tx = fpdoc.createTextNode(AcqSiteID);
  node.appendChild(tx);
  fipsnode.appendChild(node);
  return fpdoc;
# SetBIRNID();


#---------------------------------------------------------
def SetScannerID(fpdoc,ScannerID):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  nodes = fipsnode.getElementsByTagName("scannerID");
  if(len(nodes) > 0):
    fipsnode.removeChild(nodes[0]);
  node = fpdoc.createElement("scannerID");
  tx = fpdoc.createTextNode(ScannerID);
  node.appendChild(tx);
  fipsnode.appendChild(node);
  return fpdoc;
# end def

#---------------------------------------------------------
def SetProjectName(fpdoc,ProjectName):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  projnode = fipsnode.getElementsByTagName("project")[0];
  projnamenode = projnode.getElementsByTagName("name")[0];
  projnode.removeChild(projnamenode);
  node = fpdoc.createElement("name");
  tx = fpdoc.createTextNode(ProjectName);
  node.appendChild(tx);
  projnode.appendChild(node);
  return fpdoc;
# SetBIRNID();

#---------------------------------------------------------
def SetVisitID(fpdoc,VisitID):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  visitnode = fipsnode.getElementsByTagName("visit")[0];
  visitidnode = visitnode.getElementsByTagName("ID")[0];
  visitnode.removeChild(visitidnode);
  node = fpdoc.createElement("ID");
  tx = fpdoc.createTextNode("%d" % VisitID);
  node.appendChild(tx);
  visitnode.appendChild(node);
  return fpdoc;
# SetBIRNID();

#---------------------------------------------------------
def SetVisitDate(fpdoc,VisitDate):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  visitnode = fipsnode.getElementsByTagName("visit")[0];
  visitdatenode = visitnode.getElementsByTagName("visitDate")[0];
  visitnode.removeChild(visitdatenode);
  node = fpdoc.createElement("visitDate");
  tx = fpdoc.createTextNode(VisitDate);
  node.appendChild(tx);
  visitnode.appendChild(node);
  return fpdoc;
# SetBIRNID();

#---------------------------------------------------------
def SetSeriesTime(fpdoc,Seriestime):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  seriesnode = fipsnode.getElementsByTagName("series")[0];
  seriestimenode = seriesnode.getElementsByTagName("seriesTime")[0];
  seriesnode.removeChild(seriestimenode);
  node = fpdoc.createElement("seriestime");
  tx = fpdoc.createTextNode(Seriestime);
  node.appendChild(tx);
  seriesnode.appendChild(node);
  return fpdoc;
# SetBIRNID();

#---------------------------------------------------------
def SetSliceOrder(fpdoc,SOString):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  seriesnode = fipsnode.getElementsByTagName("series")[0];
  sonode     = seriesnode.getElementsByTagName("sliceorder")[0];
  seriesnode.removeChild(sonode);
  node = fpdoc.createElement("sliceorder");
  tx = fpdoc.createTextNode(SOString);
  node.appendChild(tx);
  seriesnode.appendChild(node);
  return fpdoc;
# SetBIRNID();

#---------------------------------------------------------
def SetSeriesType(fpdoc,SeriesType):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  seriesnode = fipsnode.getElementsByTagName("series")[0];
  typenode   = seriesnode.getElementsByTagName("type")[0];
  seriesnode.removeChild(typenode);
  node = fpdoc.createElement("type");
  tx = fpdoc.createTextNode(SeriesType);
  node.appendChild(tx);
  seriesnode.appendChild(node);
  return fpdoc;
# end deff

#---------------------------------------------------------
# FIPS-PROCESS.XML  operations --------------------------

def GetAnaLevelNodes(fpdoc):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  seriesnode = fipsnode.getElementsByTagName("series")[0];
  analevelnodes = seriesnode.getElementsByTagName("analysislevel");
  return analevelnodes;
#end def GetAnaNode(fpdoc):

def SetAnaLevel(fpdoc,AnaLevel):
  """
  SetAnaLevel(fpdoc,AnaLevel)
    fpdoc - fips-process.xml dom document
    AnaLevel - new analysis level
  """
  #analevelnodes = fpdoc.getElementsByTagName("analysislevel");
  analevelnodes = GetAnaLevelNodes(fpdoc);
  if(len(analevelnodes) > 0):
    analevelnode = analevelnodes[0];
    levelnodes = analevelnode.getElementsByTagName("level");
    if(len(levelnodes) > 0):
      analevelnode.removeChild(levelnodes[0]);
  else:
    seriesnode = fpdoc.getElementsByTagName("series")[0];
    analevelnode = fpdoc.createElement("analysislevel");
    seriesnode.appendChild(analevelnode);
  #endif    
  levelnode = fpdoc.createElement("level");
  txlevel = fpdoc.createTextNode(AnaLevel);
  levelnode.appendChild(txlevel);
  analevelnode.appendChild(levelnode);
  return fpdoc;
# def SetAnaLevel()

#---------------------------------------------------------
def SetAnaName(fpdoc,AnaName):
  #analevelnodes = fpdoc.getElementsByTagName("analysislevel");
  analevelnodes = GetAnaLevelNodes(fpdoc);
  if(len(analevelnodes) > 0):
    analevelnode = analevelnodes[0];
    namenodes = analevelnode.getElementsByTagName("name");
    if(len(namenodes) > 0):
      analevelnode.removeChild(namenodes[0]);
  else:
    seriesnode = fpdoc.getElementsByTagName("series")[0];
    analevelnode = fpdoc.createElement("analysislevel");
    seriesnode.appendChild(analevelnode);
  #endif    
  namenodes = analevelnode.getElementsByTagName("name");
  if(len(namenodes) > 0):
    analevelnode.removeChild(namenodes[0]);
  namenode = fpdoc.createElement("name");
  txname = fpdoc.createTextNode(AnaName);
  namenode.appendChild(txname);
  analevelnode.appendChild(namenode);
  return fpdoc;
# def SetAnaLevel()

#---------------------------------------------------------
def SetAnaDate(fpdoc,AnaDate):
  (adate,atime) = string.split(AnaDate);
  #analevelnode = fpdoc.getElementsByTagName("analysislevel")[0];
  analevelnodes = GetAnaLevelNodes(fpdoc);
  analevelnode = analevelnodes[0];
  datenodes = analevelnode.getElementsByTagName("date");
  if(len(datenodes) > 0):
    analevelnode.removeChild(datenodes[0]);
  datenode = fpdoc.createElement("date");
  txdate = fpdoc.createTextNode(adate);
  datenode.appendChild(txdate);
  analevelnode.appendChild(datenode);

  timenodes = analevelnode.getElementsByTagName("time");
  if(len(timenodes) > 0):
    analevelnode.removeChild(timenodes[0]);
  timenode = fpdoc.createElement("time");
  txtime = fpdoc.createTextNode(atime);
  timenode.appendChild(txtime);
  analevelnode.appendChild(timenode);

  return fpdoc;
# def SetAnaLevel()

#---------------------------------------------------------
def SetParadigm(fpdoc,Paradigm):
  seriesnode = fpdoc.getElementsByTagName("series")[0];
  parnodes = seriesnode.getElementsByTagName("paradigm");
  if(len(parnodes) > 0):
    seriesnode.removeChild(parnodes[0]);
  parnode = fpdoc.createElement("paradigm");
  txpar = fpdoc.createTextNode(Paradigm);
  parnode.appendChild(txpar);
  seriesnode.appendChild(parnode);
  return fpdoc;
# def SetAnaLevel()

#---------------------------------------------------------
def SetStudyName(fpdoc,StudyName):
  studynodes = fpdoc.getElementsByTagName("study");
  if(len(studynodes) > 0):
    studynode = studynodes[0];
    namenodes = studynode.getElementsByTagName("name");
    if(len(namenodes) > 0):
      studynode.removeChild(namenodes[0]);
  else:
    fipsnode = fpdoc.getElementsByTagName("fips")[0];
    studynode = fpdoc.createElement("study");
    fips.appendChild(studynode);
  #endif    
  namenodes = studynode.getElementsByTagName("name");
  if(len(namenodes) > 0):
    studynode.removeChild(namenodes[0]);
  namenode = fpdoc.createElement("name");
  txname = fpdoc.createTextNode(StudyName);
  namenode.appendChild(txname);
  studynode.appendChild(namenode);
  return fpdoc;
# def SetStudyName()

#---------------------------------------------------------
def SetStudyId(fpdoc,StudyId):
  studynodes = fpdoc.getElementsByTagName("study");
  if(len(studynodes) > 0):
    studynode = studynodes[0];
    idnodes = studynode.getElementsByTagName("ID");
    if(len(idnodes) > 0):
      studynode.removeChild(idnodes[0]);
  else:
    fipsnode = fpdoc.getElementsByTagName("fips")[0];
    studynode = fpdoc.createElement("study");
    fips.appendChild(studynode);
  #endif    
  idnodes = studynode.getElementsByTagName("ID");
  if(len(idnodes) > 0):
    studynode.removeChild(idnodes[0]);
  idnode = fpdoc.createElement("ID");
  txid = fpdoc.createTextNode(StudyId);
  idnode.appendChild(txid);
  studynode.appendChild(idnode);
  return fpdoc;
# def SetStudyId()

#---------------------------------------------------------
def SetImportStatus(fpdoc,ImportStatus):
  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];
  importnodes = fipsnode.getElementsByTagName("ImportStatus");
  if(len(importnodes) > 0):
    fipsnode.removeChild(importnodes[0]);
  importnode = fpdoc.createElement("ImportStatus");
  txImportStatus = fpdoc.createTextNode("%d"%ImportStatus);
  importnode.appendChild(txImportStatus);
  fipsnode.appendChild(importnode);
  return fpdoc;
# def SetImportStatus

#---------------------------------------------------------
def SetIsRunning(fpdoc,IsRunning,ProcId):
  #analevelnode = fpdoc.getElementsByTagName("analysislevel")[0];
  analevelnodes = GetAnaLevelNodes(fpdoc);
  analevelnode = analevelnodes[0];
  isrnodes  = analevelnode.getElementsByTagName("IsRunning");
  if(len(isrnodes) != 0):
    analevelnode.removeChild(isrnodes[0]);
  if(IsRunning == 0): return fpdoc;
    
  tx = fpdoc.createTextNode(ProcId);
  procidnode  = fpdoc.createElement("procid");
  procidnode.appendChild(tx);

  tx = fpdoc.createTextNode(os.uname()[1]);
  unamenode = fpdoc.createElement("uname");
  unamenode.appendChild(tx);

  tx = fpdoc.createTextNode("%s" % datetime.datetime.now())
  datenode = fpdoc.createElement("procdate");
  datenode.appendChild(tx);

  try:
    user = os.getlogin();
  except:
    user = "unknown";
  #endtry
  tx  = fpdoc.createTextNode(user); 
  usernode  = fpdoc.createElement("user");
  usernode.appendChild(tx);

  isrnode = fpdoc.createElement("IsRunning");
  isrnode.appendChild(procidnode);
  isrnode.appendChild(datenode);
  isrnode.appendChild(unamenode);
  isrnode.appendChild(usernode);

  analevelnode.appendChild(isrnode);
  return fpdoc;
# def SetIsRunning

#---------------------------------------------------------
def SetErrorStatus(fpdoc,ErrorStatus):
  analevelnode = fpdoc.getElementsByTagName("analysislevel")[0];
  errnodes  = analevelnode.getElementsByTagName("ErrorStatus");
  if(len(errnodes) != 0):
    analevelnode.removeChild(errnodes[0]);
  tx = fpdoc.createTextNode("%d" % ErrorStatus);
  errnode = fpdoc.createElement("ErrorStatus");
  errnode.appendChild(tx);
  analevelnode.appendChild(errnode);
  return fpdoc;
# def SetIsRunning

#---------------------------------------------------------
def SetRunNo(fpdoc,RunNo):
  seriesnode = fpdoc.getElementsByTagName("series")[0];
  runnodes  = seriesnode.getElementsByTagName("number");
  if(len(runnodes) != 0): seriesnode.removeChild(runnodes[0]);

  tx = fpdoc.createTextNode(RunNo);
  runnode = fpdoc.createElement("number");
  runnode.appendChild(tx);
  seriesnode.appendChild(runnode);
  return fpdoc;
# def 

#---------------------------------------------------------
def SetDisAcqs(fpdoc,DisAcqs):
  seriesnode = fpdoc.getElementsByTagName("series")[0];
  daqnodes  = seriesnode.getElementsByTagName("discardAcq");
  if(len(daqnodes) != 0): seriesnode.removeChild(daqnodes[0]);
  daqnodes  = seriesnode.getElementsByTagName("skipInitialVols");
  if(len(daqnodes) != 0): seriesnode.removeChild(daqnodes[0]);

  tx = fpdoc.createTextNode(DisAcqs);
  daqnode = fpdoc.createElement("skipInitialVols");
  daqnode.appendChild(tx);
  seriesnode.appendChild(daqnode);
  return fpdoc;
# def 

#---------------------------------------------------------
def SetSeriesDataFile(fpdoc,DataFile):
  seriesnode = fpdoc.getElementsByTagName("series")[0];
  fnodes  = seriesnode.getElementsByTagName("DataFile");
  if(len(fnodes) != 0): seriesnode.removeChild(fnodes[0]);

  tx = fpdoc.createTextNode(DataFile);
  fnode = fpdoc.createElement("DataFile");
  fnode.appendChild(tx);
  seriesnode.appendChild(fnode);
  return fpdoc;
# def 

#---------------------------------------------------------
def GetChildVal(node):
  for child in node.childNodes:
    tagval = string.strip(child.nodeValue);
    if(len(tagval) != 0): return tagval;
  #endfor
  return "";
#end def GetChildVal(node):

#---------------------------------------------------------
def XML2DBEntryX(xmlfile):
  """
  entryx = XML2DBEntryX(xmlfile)
  Extended entry info. Needed for fBIRN hiearchy placement
  """
  reader = Sax2.Reader();
  try:
    fpdoc = reader.fromStream(xmlfile);
  except:
    print "ERROR: reading %s as xml" % xmlfile;
    return "";
  #end
  fipsnode  = fpdoc.getElementsByTagName("FIPS")[0];
  seriesnode       = fipsnode.getElementsByTagName("series")[0];
  nameStandardNode = seriesnode.getElementsByTagName("nameStandard")[0];
  nameStandard     = fips.GetChildVal(nameStandardNode);
  projnode   = fipsnode.getElementsByTagName("project")[0];
  projidnode = projnode.getElementsByTagName("ID")[0];
  projid     = fips.GetChildVal(projidnode);

  entryx = (nameStandard,projid);
  return(entryx);
# def XML2DBEntryX(xmlfile):
#---------------------------------------------------------
def XML2DBEntry(xmlfile):
  """
  entry = XML2DBEntry(xmlfile). entry is 1x19 (does not have Nth as
  first element).
  """

  xmldir = os.path.dirname(xmlfile);
  if(len(xmldir) == 0): xmldir = '.'; # Empty if cur dir
  xmldir = fips.TrueFullPath(xmldir);

  # Create a reader to read in xml
  reader = Sax2.Reader();

  # Read in the fips process
  try:
    fpdoc = reader.fromStream(xmlfile);
  except:
    print "ERROR: reading %s as xml" % xmlfile;
    return "";
  #end

  fipsnode = fpdoc.getElementsByTagName("FIPS")[0];

  importednodes = fipsnode.getElementsByTagName("ImportStatus");
  if(len(importednodes)==0):
    importstatus = 0;
  else:
    importstatus = string.atoi(fips.GetChildVal(importednodes[0]));
  #endif
  
  bidnode = fipsnode.getElementsByTagName("BIRNID")[0];
  bid = fips.GetChildVal(bidnode);

  # Recruite Site
  if(len(bid) >= 4):
    recsite = bid[0:4];
  else:
    recsite = bid[0:len(bid)];
  #endif
  
  projnode = fipsnode.getElementsByTagName("project")[0];
  projnamenode = projnode.getElementsByTagName("name")[0];
  projname = fips.GetChildVal(projnamenode);
  
  scannernodes = fipsnode.getElementsByTagName("scannerID");
  if(len(scannernodes) > 0):
    acqsite = fips.GetChildVal(scannernodes[0]);
    if(acqsite == "uci" and projname == "FBIRNSubject2007TW"):
      acqsite = "0009"; # hardcoded for backwards compat
    #endif    
  else:
    acqsitenode = fipsnode.getElementsByTagName("acquisitionSiteID")[0];
    acqsite     = fips.GetChildVal(acqsitenode);
  #endif    
  
  visitnode = fipsnode.getElementsByTagName("visit")[0];
  visitnamenode = visitnode.getElementsByTagName("name")[0];
  visitname = fips.GetChildVal(visitnamenode);
  # May need to remove "L" before visit id.
  visitidnode = visitnode.getElementsByTagName("ID")[0];

  visitid = string.atoi(fips.GetChildVal(visitidnode));

  visitdatenodes = visitnode.getElementsByTagName("visitDate");
  if(len(visitdatenodes) > 0):
    visitdatenode = visitdatenodes[len(visitdatenodes)-1]; # get the last one
    visitdate = fips.GetChildVal(visitdatenode);
  else:
    visitdate = "1901-01-01"; # unknown
  #endif
  
  studynode     = fipsnode.getElementsByTagName("study")[0];
  studynamenode = studynode.getElementsByTagName("name")[0];
  studyname     = fips.GetChildVal(studynamenode);
  studyidnode   = studynode.getElementsByTagName("ID")[0];
  studyid       = fips.GetChildVal(studyidnode);
  # StudyTime not used yet.
  studytimenodes = studynode.getElementsByTagName("studyTime");
  if(len(studytimenodes) > 0):
    studytimenode = studytimenodes[0];
    studytime = fips.GetChildVal(studytimenode);
  else:
    studytime = "00:00:00";
  #endif
  
  seriesnode     = fipsnode.getElementsByTagName("series")[0];
  seriesnumnodes = seriesnode.getElementsByTagName("number");
  seriesnum = 0;
  if(len(seriesnumnodes) > 0):
    if(len(fips.GetChildVal(seriesnumnodes[0])) > 0):
      seriesnum = string.atoi(fips.GetChildVal(seriesnumnodes[0]));
  if(seriesnum == 0): seriesnum = 1;
  seriestypenode = seriesnode.getElementsByTagName("type")[0];
  seriestype     = fips.GetChildVal(seriestypenode);
  # SeriesTime not used yet.
  seriestimenodes = seriesnode.getElementsByTagName("seriesTime");
  if(len(seriestimenodes) > 0):
    seriestimenode = seriestimenodes[0];
    seriestime = fips.GetChildVal(seriestimenode);
  else:
    seriestime = "00:00:00";
  #endif
  
  parnode    = seriesnode.getElementsByTagName("paradigm")[0];
  paradigm   = fips.GetChildVal(parnode);
  if(paradigm == "b0"):
    nameStandardNode = seriesnode.getElementsByTagName("nameStandard")[0];
    nameStandard = fips.GetChildVal(nameStandardNode);
    if(nameStandard == "B0_mag1"   or nameStandard == "B0_mag2"):   paradigm = "b0mag";
    if(nameStandard == "B0_phase1" or nameStandard == "B0_phase2"): paradigm = "b0phase";
    if(nameStandard == "B0_real1"  or nameStandard == "B0_real2"):  paradigm = "b0real";
    if(nameStandard == "B0_imag1"  or nameStandard == "B0_imag2"):  paradigm = "b0imag";
  #endif  
  if(paradigm == "t1"):
    nameStandardNode = seriesnode.getElementsByTagName("nameStandard")[0];
    nameStandard = fips.GetChildVal(nameStandardNode);
    if(nameStandard == "t1_deface"): paradigm = "t1_deface";
  #endif  

  #ananodes  = seriesnode.getElementsByTagName("analysislevel");
  ananodes = GetAnaLevelNodes(fpdoc);
  if(len(ananodes) > 0):
    ananode = ananodes[0];

    analevelnodes  = ananode.getElementsByTagName("level");
    if(len(analevelnodes) > 0):
      analevelnode = analevelnodes[0];
      analevel = fips.GetChildVal(analevelnode);
    else:
      analevel = "none"
    #endif

    ananamenode   = ananode.getElementsByTagName("name")[0];
    ananame       = fips.GetChildVal(ananamenode);
  
    anadatenodes  = ananode.getElementsByTagName("date");
    if(len(anadatenodes) == 0):
      anadate = "unknown";
    else: anadate = fips.GetChildVal(anadatenodes[0]);
    
    anatimenodes  = ananode.getElementsByTagName("time");
    if(len(anatimenodes) == 0):
      anatime = "unknown";
    else: anatime = fips.GetChildVal(anatimenodes[0]);

    errnodes      = ananode.getElementsByTagName("ErrorStatus");
    if(len(errnodes) == 0):
      errorstatus = 0;
    else: errorstatus = string.atoi(fips.GetChildVal(errnodes[0]));
    
    isrnodes = ananode.getElementsByTagName("IsRunning");
    if(len(isrnodes) == 0):
      isrunning = 0;
    else:  isrunning = 1;
  else:
    analevel = "none";
    ananame = os.path.basename(xmldir);
    anadate = "unknown";
    anatime = "unknown";
    errorstatus = 0;
    isrunning = 0;
  #endif    

  h = (bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
       visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
       anatime,errorstatus,isrunning,xmldir);
  #pdb.set_trace();  
  return h;
#end def XML2DBEntry()

# END FIPS-PROCESS.XML  operations --------------------------
#---------------------------------------------------------------------

#---------------------------------------------------------------------
def PrintDBPreamble(fp):
  # For stdout: PrintDBPreamble(sys.stdout):
  fp.write("# DO NOT EDIT - THIS FILE IS AUTOMATICALLY GENERATED\n");
  fp.write("# DATASUMDATE %s \n" % datetime.datetime.now());
  fp.write("# FIPS_ROOT %s\n" % os.environ.get("FIPS_ROOT",0));
  fp.write("# 20 columns\n" );
  fp.write("# ColId  1 Nth\n");
  fp.write("# ColId  2 BIRNID\n");
  fp.write("# ColId  3 RecruitSite\n");
  fp.write("# ColId  4 AcqSite\n");
  fp.write("# ColId  5 Project \n");
  fp.write("# ColId  6 StudyName\n");
  fp.write("# ColId  7 StudyId\n");
  fp.write("# ColId  8 VisitName\n");
  fp.write("# ColId  9 VisitId\n");
  fp.write("# ColId 10 VisitDate\n");
  fp.write("# ColId 11 SeriesType\n");
  fp.write("# ColId 12 Paradgim\n");
  fp.write("# ColId 13 SeriesNumber\n");
  fp.write("# ColId 14 AnalysisLevel\n");
  fp.write("# ColId 15 AnalysisName\n");
  fp.write("# ColId 16 AnalysisDate\n");
  fp.write("# ColId 17 AnalysisTime\n");
  fp.write("# ColId 18 ErrorStatus\n");
  fp.write("# ColId 19 IsRunning\n");
  fp.write("# ColId 20 XMLDir\n");
  fp.write("# \n");
  fp.write("# Nth BIRNId RecSite AcqSite Proj StudyName StudyId VisitName VisitId VisitDate RunType Paradigm RunNo AnalysisLevel AnalysisName AnalysisDate AnalysisTime ErrorStatus IsRunning XMLDir\n");
  return;
#def PrintDBPreamble(fp):  

#---------------------------------------------------------------------
def DBMatchEntryExact(entry,db):
  """
  indlist = DBMatchEntryExact(entry,db);
  Returns indicies to exact matches, or [] if no match. Note that entry
  and db have to have the same number of columns, but they can be any
  number (ie, don't have to be 19 or 20).
  """
  ind = [];
  nth = 0;
  for e in db:
    if(e == entry): ind.append(nth);
    nth = nth + 1;
  #end for e
  return ind;
#def DBMatchEntryExact(entry,db)

#---------------------------------------------------------------------
def FindXML(*args):
  """
  xmllist = FindXML(top,<xmllist>) - returns list of paths of 
  fips-process.xml under top. Follows symlinks recursively. The
  paths are absolute.
  """
  top = args[0];
  if(len(args) == 2):
    xmllist = args[1];
  else:
    xmllist = [];
  #endif
  
  # Begin loop thru tree
  for (root, dirs, files) in os.walk(top):

    # Go through each file
    for f in files:
      if(f == "fips-process.xml"):
        # If this file is fips-process.xml, construct full
        # path name and append to list
        fpath = os.path.abspath(os.path.join(root, f));
        xmllist.append(fpath);
        # print "%s" % (fpath);
      #endif
    #end for f

    # Go through each directory, recursive for symlinks
    for d in dirs:
      dpath = os.path.join(root, d);
      if(os.path.islink(dpath)):
        #print "recursive %s" % dpath;
        xmllist = fips.FindXML(dpath,xmllist);
      #endif
    #end for d

  #end for walk
  return xmllist;
#end function findfpxml(top,xmllist):

#---------------------------------------------------------------------
def FindDB(TopDBDir):
  """
  dblist = FindDB(TopDBDir) - returns list of paths of fips.database.dat
  under TopDBDir. Follows symlinks recursively. The  paths are absolute.
  """
  dblist = [];
  # Begin loop thru tree
  for (root, dirs, files) in os.walk(TopDBDir):

    # Go through each file
    for f in files:
      if(f == "fips.database.dat"):
        # If this file is fips.database.dat, construct full
        # path name and append to list
        fpath = os.path.abspath(os.path.join(root,f));
        dblist.append(fpath);
        # print "%s" % (fpath);
      #endif
    #end for f

    # Go through each directory, recursive for symlinks
    for d in dirs:
      dpath = os.path.join(root, d);
      if(os.path.islink(dpath)):
        #print "recursive %s" % dpath;
        dblist = fips.FindDB(dpath,dblist);
      #endif
    #end for d

  #end for walk
  return dblist;
#end function FindDB(TopDBDir)

#---------------------------------------------------------------------
def BuildDB(*topdir):
  """
  db = BuildDB(<topdir>);
  Returns Nx20 db matrix based on xml files found under topdir.
  If topdir is not specified, then FIPS_ROOT is used. The 1st
  column is the 1-based Nth file.
  """
  if(len(topdir) == 0):
    topdir = os.environ.get('FIPS_ROOT',0);
  else:
    topdir = topdir[0];
  #endif

  xmllist = fips.FindXML(topdir);
  if(len(xmllist) == 0): return [];

  db = [];
  nth = 1;
  for xml in xmllist:
    e = list(XML2DBEntry(xml));
    e.insert(0,nth);
    db.append(e);
    nth = nth + 1;
    #pdb.set_trace();  
  #end for

  return db;
#def BuildDB(*topdir):

#---------------------------------------------------------------------
def MergeDB(db1,db2):
  """
  db = MergeDB(db1,db2)
  Returns Nx20 db matrix by merging. Does not remove duplicates.
  """
  db = [];
  #pdb.set_trace();  
  for entry in db1: db.append(entry);
  for entry in db2: db.append(entry);
  return db;
# def MergeDB()

#---------------------------------------------------------------------
def UniqueDB(db):
  """
  db = UniqueDB(db);
  Removes duplicates from db. db is Nx20.  Does not use Nth or XMLFile.
  Renumbers the 1st col. Returned db is also Nx20.  Does not imply 
  hierarchy uniqueness.
  """
  dbu = [];
  nth = 1;
  for entry in db:
    # remove the 1st (Nth) and last (xmldir) cols
    entry2 = entry[1:len(entry)-1];
    xmldir = entry[len(entry)-1];
    entry_use = entry;
    hit = 0;
    for uentry in dbu:
      uentry2 = uentry[1:len(uentry)-1];
      uxmldir = entry[len(entry)-1];
      if(entry2 == uentry2):
        hit = 1;
        if(entry[0] != uentry[0]):
          # print "Input entry %s redundant with %s" % (uentry[0],entry[0])
          # Try to resolve redundancy by seeing whether only one exists
          if(os.path.exists(uxmldir) and not os.path.exists(xmldir)):
            entry_use = uentry;
          #endif
        break;
      #endif
    #endfor
    if(not hit):
      entry_use[0] = nth;
      dbu.append(entry_use);
      nth = nth + 1;
    #endif
  #endfor
  return dbu;
#def UniqueDB()

#---------------------------------------------------------------------
def HEntryExists(entry,db):
  """
  100|200 = HEntryExists(entry,db);
  Checks whether entry matches any in the DB based on where it
  would land in the hierarchy. Returns 100 if it does NOT exist,
  or 200 if it does.
  """
  hentry = fips.Entry2HEntry(entry);
  hit = 0;
  for dbentry in db:
    dbhentry = fips.Entry2HEntry(dbentry);
    if(hentry == dbhentry): return 200;
  #end for
  return 100;
# HEntryExists(entry,db)

#---------------------------------------------------------------------
def FindDBDuplicates(db):
  """
  dups = FindDBDuplicates(db);
  Creates a list of duplicates. Two entries are duplicates if they
  resolve to the same location in the hierarchy but exist in two
  different places.
  """
  udb = fips.HUniqueDB(db);
  dups = [];
  nth = 0;
  for entry1 in udb:
    hentry1 = fips.Entry2HEntry(entry1);
    duplist = [];
    for entry2 in db:
      if(entry1 == entry2): continue;
      hentry2 = fips.Entry2HEntry(entry2);
      if(hentry1 == hentry2):
        xmldir1 = entry1[len(entry1)-1];
        xmldir2 = entry2[len(entry2)-1];
        if(xmldir1 != xmldir2):
          if(len(duplist)==0): duplist.append(xmldir1);
          duplist.append(xmldir2);
        #endif  
      #endif
    #endfor
    if(len(duplist) > 0): dups.append(duplist);
    nth = nth + 1;
  #endfor
  return(dups);
# def FindDBDuplicates(db)

#---------------------------------------------------------------------
def HUniqueDB(db):
  """
  db = HUniqueDB(db);
  Removes duplicates from db based on hierarchy uniqueness.
  Renumbers the 1st col. Returned db is also Nx20.
  """
  dbu = [];
  nth = 1;
  for entry in db:
    entry2 = fips.Entry2HEntry(entry);
    hit = 0;
    for uentry in dbu:
      uentry2 = fips.Entry2HEntry(uentry);
      #pdb.set_trace();
      if(entry2 == uentry2):
        hit = 1;
        if(entry[0] != uentry[0]):
          print "Input entry %s redundant with %s" % (entry[0],uentry[0])
        break;
      #endif
    #endfor
    if(not hit):
      entry[0] = nth;
      dbu.append(entry);
      nth = nth + 1;
    #endif
  #endfor
  return dbu;
#def HUniqueDB()

#---------------------------------------------------------------------
def HVisitUniqueDB(db):
  """
  db = HVisitUniqueDB(db);
  Removes duplicates from db based on hierarchy 'visit' uniqueness.
  This just means that everything below the visit is ignored, ie,
  paradigm, run/series, analevel, and ananame is ignored. Returns
  entry corresponding to the first run in a paradigm.
  Renumbers the 1st col. Returned db is also Nx20.
  """
  dbu = [];
  nth = 1;
  for entry in db:
    entry2 = list(fips.Entry2HEntry(entry));
    #pdb.set_trace();
    # Delete everything below the visit
    del entry2[10]; # Delete AnalysisName (11th in HEntry)
    del entry2[9]; # Delete AnalysisLevel (10th in HEntry)
    del entry2[8]; # Delete Run/Series Number (9th in HEntry)
    del entry2[7]; # Delete Paradigm (8th in HEntry)
    entry2 = tuple(entry2);
    hit = 0;
    for uentry in dbu:
      uentry2 = list(fips.Entry2HEntry(uentry));
      # Delete everything below the visit
      del uentry2[10]; # Delete AnalysisName (11th in HEntry)
      del uentry2[9]; # Delete AnalysisLevel (10th in HEntry)
      del uentry2[8]; # Delete Run/Series Number (9th in HEntry)
      del uentry2[7]; # Delete Paradigm (8th in HEntry)
      uentry2 = tuple(uentry2);
      if(entry2 == uentry2):
        hit = 1;
        break;
      #endif
    #endfor
    if(not hit):
      entry[0] = nth;
      dbu.append(entry);
      nth = nth + 1;
    #endif
  #endfor
  return dbu;
#def HVisitUniqueDB()

#---------------------------------------------------------------------
def DBMatch(match,db):
  """
  dbmatches = DBMatch(match,db);
  match - 1x20 db entry. Elements that are empty are not used as match criteria.
  db - Nx20.
  dbmatches - Mx20 list of matches.
  Does not match on xml file.
  """
  (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
   visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
   anatime,errorstatus,isrunning,xmldir) = match;

  dbmatch = [];
  for entry in db:
    (Enth,Ebid,Erecsite,Eacqsite,Eprojname,Estudyname,Estudyid,Evisitname,Evisitid,
     Evisitdate,Eseriestype,Eparadigm,Eseriesnum,Eanalevel,Eananame,Eanadate,
     Eanatime,Eerrorstatus,Eisrunning,Exmldir) = entry;
    if(len(nth)         != 0   and   nth         != Enth):         continue;
    if(len(bid)     > 0 and bid.count(Ebid)         == 0): continue;
    if(len(recsite) > 0 and recsite.count(Erecsite) == 0): continue;
    if(len(acqsite) > 0 and acqsite.count(Eacqsite) == 0): continue;
    if(len(projname)    != 0   and   projname    != Eprojname):    continue;
    if(len(studyname)   != 0   and   studyname   != Estudyname):   continue;
    if(len(studyid)     != 0   and   studyid     != Estudyid):     continue;
    if(len(visitname)   != 0   and   visitname   != Evisitname):   continue;
    if(len(visitid) > 0 and visitid.count(Evisitid) == 0): continue;
    if(len(visitdate)   != 0   and   visitdate   != Evisitdate):   continue;
    if(len(seriestype)  != 0   and   seriestype  != Eseriestype):  continue;
    if(len(paradigm)    != 0   and   paradigm    != Eparadigm):    continue;
    if(len(seriesnum) > 0 and seriesnum.count(Eseriesnum) == 0): continue;
    if(len(analevel)    != 0   and   analevel    != Eanalevel):    continue;
    if(len(ananame)     != 0   and   ananame     != Eananame):     continue;
    if(len(anadate)     != 0   and   anadate     != Eanadate):     continue;
    if(len(anatime)     != 0   and   anatime     != Eanatime):     continue;
    if(errorstatus      != -1  and   errorstatus != Eerrorstatus): continue;
    if(isrunning        != -1  and   isrunning   != Eisrunning):   continue;
    dbmatch.append(entry);
  #end for
  #print "Found %d matches" % len(dbmatch);

  return dbmatch;

#def DBMatch(entry,db);

#---------------------------------------------------------------------
def DBMatchInvert(match,db):
  """
  dbmatches = DBMatchInvert(match,db);
  Inverted match. May not be that useful.
  match - 1x20 db entry. Elements that are empty are not used as match criteria.
  db - Nx20.
  dbmatches - Mx20 list of matches.
  Does not match on xml file.
  """
  (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
   visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
   anatime,errorstatus,isrunning,xmldir) = match;

  dbmatch = [];
  for entry in db:
    (Enth,Ebid,Erecsite,Eacqsite,Eprojname,Estudyname,Estudyid,Evisitname,Evisitid,
     Evisitdate,Eseriestype,Eparadigm,Eseriesnum,Eanalevel,Eananame,Eanadate,
     Eanatime,Eerrorstatus,Eisrunning,Exmldir) = entry;
    if(len(nth)         != 0   and   nth         == Enth):         continue;
    if(len(bid)     > 0 and bid.count(Ebid)         != 0): continue;
    if(len(recsite) > 0 and recsite.count(Erecsite) != 0): continue;
    if(len(acqsite) > 0 and acqsite.count(Eacqsite) != 0): continue;
    if(len(projname)    != 0   and   projname    == Eprojname):    continue;
    if(len(studyname)   != 0   and   studyname   == Estudyname):   continue;
    if(len(studyid)     != 0   and   studyid     == Estudyid):     continue;
    if(len(visitname)   != 0   and   visitname   == Evisitname):   continue;
    if(len(visitid) > 0 and visitid.count(Evisitid) != 0): continue;
    if(len(visitdate)   != 0   and   visitdate   == Evisitdate):   continue;
    if(len(seriestype)  != 0   and   seriestype  == Eseriestype):  continue;
    if(len(paradigm)    != 0   and   paradigm    == Eparadigm):    continue;
    if(len(seriesnum) > 0 and seriesnum.count(Eseriesnum) != 0): continue;
    if(len(analevel)    != 0   and   analevel    == Eanalevel):    continue;
    if(len(ananame)     != 0   and   ananame     == Eananame):     continue;
    if(len(anadate)     != 0   and   anadate     == Eanadate):     continue;
    if(len(anatime)     != 0   and   anatime     == Eanatime):     continue;
    if(errorstatus      != -1  and   errorstatus == Eerrorstatus): continue;
    if(isrunning        != -1  and   isrunning   == Eisrunning):   continue;
    dbmatch.append(entry);
  #end for
  #print "Found %d matches" % len(dbmatch);

  return dbmatch;

#def DBMatchInvert(entry,db);

#---------------------------------------------------------------------
def DBMatchStimSched(db,WantExist):
  """
  dbmatches = DBMatchStimSched(db,WantExist);
  Returns db entries in which the StimulusSchedule directory either
  exists (WantExist=1) or does not exist (WantExist=0).
  dbmatches - Mx20 list of matches.
  """
  #print "Searching %d" % len(db);
  dbmatch = [];
  nth = 1;
  for entry in db:
    (Enth,Ebid,Erecsite,Eacqsite,Eprojname,Estudyname,Estudyid,Evisitname,Evisitid,
     Evisitdate,Eseriestype,Eparadigm,Eseriesnum,Eanalevel,Eananame,Eanadate,
     Eanatime,Eerrorstatus,Eisrunning,Exmldir) = entry;
    #print "%d %s %s %d %d" % (nth,Ebid,Eacqsite,Evisitid,Eseriesnum);
    xmlfile = os.path.join(Exmldir, "fips-process.xml");
    parpath = XML2PathParent(xmlfile,"schedule","");
    scheddir = os.path.join(parpath, "StimulusSchedule");
    if(WantExist and os.path.exists(scheddir)): dbmatch.append(entry);
    if(not WantExist and not os.path.exists(scheddir)): dbmatch.append(entry);
    nth = nth + 1;
  #end for
  #print "Found %d matches" % len(dbmatch);

  return dbmatch;

#enddef DBMatchStimSched(db,WantExist);

#---------------------------------------------------------------------
def DBMissingFLAC(db,FLACName):
  """
  dbmatches = DBMissingFLAC(db,FLACName)
  Returns db entries in which there is raw data but not a FirstLevel
  matching FLACName or FLACName exists but has an error.
  dbmatches - Mx20 list of matches.
  """
  #print "Searching %d" % len(db);
  dbmatch = [];
  nth = 1;
  for entry in db:
    (Enth,Ebid,Erecsite,Eacqsite,Eprojname,Estudyname,Estudyid,Evisitname,Evisitid,
     Evisitdate,Eseriestype,Eparadigm,Eseriesnum,Eanalevel,Eananame,Eanadate,
     Eanatime,Eerrorstatus,Eisrunning,Exmldir) = entry;
    if(Eanalevel != "none"): continue;
    match = deepcopy(entry);
    match[0] = [];
    match[8] = [Evisitid];
    match[12] = [Eseriesnum];
    match[9] = '';
    match[10] = '';
    match[13] = "FirstLevel";
    match[14] = FLACName;
    match[15] = ''; # analysisdate
    match[16] = ''; # analysistime
    match[17] = 0;  # error status
    match[18] = 0;  # run status
    match[19] = '';   # xmldir
    # Check whether the analysis exists in the DB
    Ematches = fips.DBMatch(match,db);
    if(len(Ematches) != 0): continue;
    #entry[0] = nth;
    dbmatch.append(entry);
    nth = nth + 1;
  #end for

  return dbmatch;

#enddef DBMatchStimSched(db,WantExist);

#------------------------------------------------------------------
def StripAnaName(ananame):
  """
  ananame = StripAnaName(ananame);
  Removes .flac or .preproc extension from ananame. If neither of
  those are extensions, then ananame is returned.
  """

  # Check for .flac extension
  n = len(ananame);
  if(n > 5):
    tmp = ananame[n-5:n+1];
    if(tmp == ".flac"): return ananame[0:n-5];
  #endif

  # Check for .preproc extension
  if(n > 8):
    tmp = ananame[n-8:n+1];
    if(tmp == ".preproc"): return ananame[0:n-8];
  #endif

  return ananame;      
#def StripAnaName(ananame)

#------------------------------------------------------------------
def UniqueProjects(db):
  """
  projectlist = UniqueProjects(db);
  Returns a list of unique projects found in db.
  """
  plist = [];
  for entry in db: plist.append(entry[4]);

  uplist = [];
  for proj in plist:
    hit = 0;
    for uproj in uplist:
      if(proj == uproj):
        hit = 1;
        break;
      #endif
    #endfor
    if(not hit): uplist.append(proj);
  #endfor
  return uplist;
# def UniqueProjects(db)

#------------------------------------------------------------------
def UniqueParadigms(db,project):
  """
  paradigmlist = UniqueParadigms(db,project);
  Returns a list of unique paradigms for the given project found in db.
  """
  # Make a list of paradigms
  plist = [];
  for entry in db:
    eproject = entry[4];
    if(eproject != project): continue;
    plist.append(entry[11]);
  #endfor

  uplist = [];
  for paradigm in plist:
    hit = 0;
    for uparadigm in uplist:
      if(paradigm == uparadigm):
        hit = 1;
        break;
      #endif
    #endfor
    if(not hit): uplist.append(paradigm);
  #endfor
  return uplist;
# def UniqueParadigms(db):

#------------------------------------------------------------------
def UniqueBIRNIDs(db,project,paradigm):
  """
  bidlist = UniqueBIRNIDs(db,project,paradigm)
  Returns a list of unique bids for the given proj and par.
  """
  match = ['','','','','','','','',[],'','','',[],'','','','',-1,-1,''];
  match[4] = project;
  match[11] = paradigm;
  matches = fips.DBMatch(match,db);
  ubidlist = [];
  for m in matches:
    bid = m[1];
    isunique = 1;
    for ubid in ubidlist:
      if(bid == ubid):
        isunique = 0;
        break;
      #endif
    #endfor
    if(isunique): ubidlist.append(bid);
  #end
  return ubidlist;
# def UniqueBID()

#------------------------------------------------------------------
def UpdateDBEntry(newentry,db):
  """
  db = UpdateDBEntry(entry,db);
  Finds entry in db that matchs newentry. Match is based on hierarchy
  components.  Changes the others to those in newentry. If there is
  no match, then newentry is appended to the db.
  """
  db2 = deepcopy(db);
  newhentry = fips.Entry2HEntry(newentry);
  nth = 0;
  hit = 0;
  for entry in db:
    hentry = fips.Entry2HEntry(entry);
    if(newhentry == hentry):
      db2[nth] = newentry;
      hit = 1;
      #print "UpdateDBEntry: hit"
    #endif
    nth = nth + 1;
  #endfor
  if(hit==0): db2.append(newentry);
  return(db2);
#end UpdateDBEntry(newentry,db)

# --------------------------------------------------------------
def Entry2HEntry(entry):
  """
  hentry = Entry2HEntry(entry)
  Returns the hierarchy components of entry.
  """
  (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
   visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
   anatime,errorstatus,isrunning,xmldir) = entry;
  hentry = [bid,acqsite,projname,studyname,studyid,visitname,visitid,
            paradigm,seriesnum,analevel,ananame];
  return(hentry);
#def Entry2HEntry(entry):

# --------------------------------------------------------------
def TrueFullPath(path0):
  """
  tfpath = TrueFullPath(path);
  Returns absolute path within the mount point tree. If the path
  does not exist, returns path.
  """
  path = deepcopy(path0);
  if(not os.path.exists(path)):
    #print "WARNING: fips.TrueFullPath(): %s does not exist" % path;
    return path;
  #endif
  base = '';
  if(os.path.isfile(path)):
    # Input is a file
    base = os.path.basename(path);
    path = os.path.dirname(path);
    if(len(path)==0): path = ".";
  #endif
  pwd = os.getcwd(); # keep current dir
  os.chdir(path);    # cd to path
  path = os.path.abspath("."); # get abs path
  if(len(base) > 0): path = os.path.join(path, base);
  os.chdir(pwd); # cd back to current dir
  return(path);
#end  TrueFullPath(path)

# --------------------------------------------------------------
def XML2PathParent(xmlfile,pathtype,altroot):
  """
  path = XML2PathParent(xmlfile,pathtype,altroot);
  Returns the path or parent path in the hierarchy.
  pathtype can be:
    flac, preproc, schedule, raw, xRunFFx, xRunRFx, xRunMFx, xVisitFFx - returns parent
    visit - returns visit dir
  """
  (bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
   visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
   anatime,errorstatus,isrunning,xmldir) = XML2DBEntry(xmlfile);

  ns = os.environ.get('FIPS_NAMESPACE',0);
  if(ns == 0): ns = "FIPS"; # Use FIPS namespace by default
  #if(ns == 0): ns = "FBIRN"; # Use FBIRN namespace by default
  if(ns != "FBIRN" and ns != "FIPS"):
    print "ERROR: FIPS_NAMESPACE = %s, must be FIPS or FBIRN" % (ns);
    return ();
  #endif
  if(ns == "FBIRN"): (nameStandard,projid) = fips.XML2DBEntryX(xmlfile);

  ddir = "";
  if(len(altroot) == 0):
    fipsroot = os.environ.get('FIPS_ROOT',0);
  else:
    if(altroot == "srb"):
      fipsroot = "/home/Projects";
      ddir = "Data";
      if(acqsite == "ucsd-e" or acqsite == "ucsd-w"): acqsite = "0008";
    else:
      fipsroot = altroot;
    #endif    
  #endif    
    
  if(pathtype == "flac" or pathtype == "preproc" or pathtype == "schedule"):
    if(ns == "FBIRN"):
      # Eg: 000640456777/scanVisit__0006__0002/MRI__0001/sirp3/Analysis/Original__0001
      projseg = "%s__%s" % (projname,projid);
      scanseg = "%s__%s__%04d" % (visitname,acqsite,visitid);
      studyseg = "%s__%s" % (studyname,studyid);
      path = os.path.join(fipsroot,projseg,ddir,bid,scanseg,studyseg,
                          nameStandard,"Analysis","Original__0001");
      return(path);
    else:
      # Eg: 000640456777/0006/v0002/sirp/r0003
      visitseg = "v%04d" % (visitid);
      runseg   = "r%03d" % (seriesnum);
      path = os.path.join(fipsroot,projname,bid,acqsite,visitseg,paradigm,runseg);
      return(path);
    #endif
  #endif
  if(pathtype == "raw"):
    if(ns == "FBIRN"):
      projseg = "%s__%s" % (projname,projid);
      scanseg = "%s__%s__%04d" % (visitname,acqsite,visitid);
      studyseg = "%s__%s" % (studyname,studyid);
      path = os.path.join(fipsroot,projseg,ddir,bid,scanseg,studyseg,
                          nameStandard,"Native","Original__0001");
      return(path);
    else:
      visitseg = "v%04d" % (visitid);
      runseg   = "r%03d" % (seriesnum);
      path = os.path.join(fipsroot,projname,bid,acqsite,visitseg,paradigm,runseg);
      return(path);
    #endif
  #endif
  if(pathtype == "visit"):
    if(ns == "FBIRN"):
      projseg = "%s__%s" % (projname,projid);
      scanseg = "%s__%s__%04d" % (visitname,acqsite,visitid);
      path = os.path.join(fipsroot,projseg,ddir,bid,scanseg);
      return(path);
    else:
      visitseg = "v%04d" % (visitid);
      path = os.path.join(fipsroot,projname,bid,acqsite,visitseg);
      return(path);
    #endif
  #endif
  if(pathtype == "xRunFFx" or pathtype == "xrunffx"):
    if(ns == "FBIRN"):
      projseg = "%s__%s" % (projname,projid);
      scanseg = "%s__%s__%04d" % (visitname,acqsite,visitid);
      studyseg = "%s__%s" % (studyname,studyid);
      parffx = "%s-ffx" % (paradigm);
      path = os.path.join(fipsroot,projseg,ddir,bid,scanseg,studyseg,
                          "Analysis",parffx,"Original__0001");
      return(path);
    else:
      visitseg = "v%04d" % (visitid);
      runseg   = "xRunFFx";
      path = os.path.join(fipsroot,projname,bid,acqsite,visitseg,paradigm,runseg);
      return(path);
    #endif
  #endif
  if(pathtype == "xRunRFx" or pathtype == "xrunrfx"):
    if(ns == "FBIRN"):
      projseg = "%s__%s" % (projname,projid);
      scanseg = "%s__%s__%04d" % (visitname,acqsite,visitid);
      studyseg = "%s__%s" % (studyname,studyid);
      parffx = "%s-rfx" % (paradigm);
      path = os.path.join(fipsroot,projseg,ddir,bid,scanseg,studyseg,
                          "Analysis",parffx,"Original__0001");
      return(path);
    else:
      visitseg = "v%04d" % (visitid);
      runseg   = "xRunRFx";
      path = os.path.join(fipsroot,projname,bid,acqsite,visitseg,paradigm,runseg);
      return(path);
    #endif
  #endif
  if(pathtype == "xRunMFx" or pathtype == "xrunmfx"):
    if(ns == "FBIRN"):
      projseg = "%s__%s" % (projname,projid);
      scanseg = "%s__%s__%04d" % (visitname,acqsite,visitid);
      studyseg = "%s__%s" % (studyname,studyid);
      parffx = "%s-rfx" % (paradigm);
      path = os.path.join(fipsroot,projseg,ddir,bid,scanseg,studyseg,
                          "Analysis",parffx,"Original__0001");
      return(path);
    else:
      visitseg = "v%04d" % (visitid);
      runseg   = "xRunRFx";
      path = os.path.join(fipsroot,projname,bid,acqsite,visitseg,paradigm,runseg);
      return(path);
    #endif
  #endif
  if(pathtype == "xVisitFFx" or pathtype == "xvisitffx"):
    if(ns == "FBIRN"):
      # I'm not sure about this one.
      projseg = "%s__%s" % (projname,projid);
      parffx = "%s-ffx" % (paradigm);
      path = os.path.join(fipsroot,projseg,ddir,bid,"Analysis",parffx,"Original__0001");
      return(path);
    else:
      visitseg = "xVisitFFx";
      path = os.path.join(fipsroot,projname,bid,acqsite,visitseg,paradigm);
      return(path);
    #endif
  #endif
  return (); # Should never get here
# def XML2PathParent(xmlfile,pathtype)

# --------------------------------------------------------------
def MountPoint(path):
  """
  mountpoint = MountPoint(path);
  Returns the absolute path to the mount point of path
  """
  path = fips.TrueFullPath(path);
  while(not os.path.ismount(path)):
    path = os.path.dirname(path);
  return path;    
# def MountPoint(path)

# --------------------------------------------------------------
def DBDisks(db):
  """
  disks = DBDisks(db);
  Returns unique list of file systems/mount points that the DB spans.
  """
  alldisks = [];
  for entry in db:
    xmlfile = entry[len(entry)-1];
    alldisks.append(fips.MountPoint(xmlfile));
  #end
  udisks = [];
  for d in alldisks:
    hit = 0;
    for ud in udisks:
      if(d == ud):
        hit = 1;
        break;
      #endif
    #endfor
    if(not hit): udisks.append(d);
  #endfor
  return(udisks);
#def DBDisks(db)

# --------------------------------------------------------------
def DBDiskUsage(db):
  """
  df = DBDiskUsage(db);
  Returns unique list of file systems/mount points that the DB spans
  and the amount of usage on each.
  df = [disk disksizeG freesizeG freepct usepct];
  """
  disks = fips.DBDisks(db);
  df = [];
  for disk in disks:
    st = os.statvfs(disk);
    disksize = 1e-9*st[statvfs.F_BLOCKS]*st[statvfs.F_FRSIZE];
    freesize = 1e-9*st[statvfs.F_BAVAIL]*st[statvfs.F_FRSIZE];
    freepct  = 100*freesize/disksize;
    usepct   = 100-freepct;
    dfdisk = [disk,disksize,freesize,freepct,usepct];
    df.append(dfdisk);
  #endfor
  return(df);
#def DBDiskUsage(db)

# --------------------------------------------------------------
def PrintDBDiskUsage(*args):
  """
  PrintDBDiskUsage(db,<fp>);
  Prints disk usage as:  Disk SizeG  FreeG  FreePct UsedPct
  """
  db = args[0];
  if(len(args)>1):
    fp = args[1];
  else:
    fp = sys.stdout;
  #endif
  dfdisks = fips.DBDiskUsage(db)
  fp.write("Disk                   SizeG  FreeG   FreePct   UsedPct\n");
  for df in dfdisks:
    fp.write("%s %5.1f  %5.1f   %5.1f    %5.1f\n" % tuple(df));
  #endfor
  return;
#def PrintDBDiskUsage(db)

#----------------------------------------------------------
def LoadFSBIRNID(*arg):
  """
  fsbirnid = LoadFSBIRNID(<project>);
  Loads the FSBIRNID.dat file. fsbirnid is a list of
  BIRNID/FreeSurfer subjectname pairs (ie, the first
  item is the BIRNID and the 2nd is the subject name).
  """
  if(len(arg) > 0):
    Project = arg[0];
  else:
    Project = [];
  #end
  if(len(Project) == 0):
    # If project not specified, look in FIPS_DB_DIR 
    dbdir = os.environ.get('FIPS_DB_DIR',0);
    fsbidfile = dbfile = os.path.join(dbdir,"FSBIRNID.dat");
  else:
    # If project IS specified, look in FIPS_FLAC_DIR/Project
    # If not there, then look in FIPS_DB_DIR 
    flacdir = os.environ.get('FIPS_FLAC_DIR',0);
    fsbidfile = os.path.join(flacdir,Project,"FSBIRNID.dat");
    if(not os.path.exists(fsbidfile)):
      dbdir = os.environ.get('FIPS_DB_DIR',0);
      fsbidfile = dbfile = os.path.join(dbdir,"FSBIRNID.dat");
    #endif
  #endif
  try:
    fp = open(fsbidfile,'r');
  except:
    print "ERROR: could not open %s for reading" % (fsbidfile);
    return [];
  #end try    

  fsbirnid = [];
  for line in fp:
    ind = line.rfind('#');
    if(ind != -1): continue;
    tmp = string.split(line); # (birnid,subject)
    fsbirnid.append(tmp);
    #print "%s %s" % tuple(tmp);
  #end for line
  fp.close();
  return fsbirnid;
#end def  

#-------------------------------------------------------------
def BIRNID2FreeSurfer(birnid,fsbirnid):
  """
  subject = BIRNID2FreeSurfer(birnid,fsbirnid);
  Returns the freesurfer subject name of the given BIRNDID as found in
  the fsbirnid table. If no match is found, reutrns null.
  """
  subject = [];
  for entry in fsbirnid:
    bid = entry[0];
    if(bid == birnid):
      subject = entry[1];
      return subject;
    #endif
  #endfor
  return subject;
#end def BIRNID2FreeSurfer(birnid,fsbirnid):

#-------------------------------------------------------------
def PrintDB(*args):
  """
  PrintDB(db,PrintWhat,<fsbirnid>);

  """
  db = args[0];
  PrintWhat = args[1];
  fsbirnid = [];
  if(len(args) == 3): fsbirnid = args[2];

  fmt = "";
  for pw in PrintWhat:
    if(pw == "row"):        fmt +=  "%3s ";
    if(pw == "birnid"):     fmt +=  "%13s ";
    if(pw == "recsite"):    fmt +=  "%4s ";
    if(pw == "acqsite"):    fmt +=  "%6s "; #site, scanner
    if(pw == "project"):    fmt +=  "%20s ";
    if(pw == "visit"):      fmt +=  "%3d ";
    if(pw == "visitdate"):  fmt +=  "%10s ";
    if(pw == "study"):      fmt +=  "%6s ";
    if(pw == "studyid"):    fmt +=  "%6s ";
    if(pw == "paradigm"):   fmt +=  "%15s ";
    if(pw == "run"):        fmt +=  "%3d ";
    if(pw == "analevel"):   fmt +=  "%10s "; # analysis level
    if(pw == "ananame"):    fmt +=  "%10s "; # analysis name
    if(pw == "anadate"):    fmt +=  "%10s "; # analysis data
    if(pw == "anatime"):    fmt +=  "%10s "; # analysis time
    if(pw == "error"):      fmt +=  "%d "; 
    if(pw == "isrunning"):  fmt +=  "%d "; 
    if(pw == "xmldir"):     fmt +=  "%s "; 
    if(pw == "freesurfer"):
      if(len(fsbirnid) == 0):
        print "ERROR: PrintDB(): cannot print freesurfer subject without fsbirnid"
        return 1;
      #endif
      fmt +=  "%10s ";
    #endif      
  #end

  #print fmt;
  #pdb.set_trace();

  for entry in db:
    (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
     visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
     anatime,errorstatus,isrunning,xmldir) = entry;
    eline = [];
    for pw in PrintWhat:
      if(pw == "row"):        eline.append(nth);
      if(pw == "birnid"):     eline.append(bid);
      if(pw == "recsite"):    eline.append(recsite);
      if(pw == "acqsite"):    eline.append(acqsite); # site, scanner
      if(pw == "project"):    eline.append(projname);
      if(pw == "study"):      eline.append(studyname);
      if(pw == "studyid"):    eline.append(studyid);
      if(pw == "visit"):      eline.append(visitid);
      if(pw == "visitdate"):  eline.append(visitdate);
      if(pw == "paradigm"):   eline.append(paradigm);
      if(pw == "run"):        eline.append(seriesnum);
      if(pw == "analevel"):   eline.append(analevel);
      if(pw == "ananame"):    eline.append(ananame);
      if(pw == "anadate"):    eline.append(anadate);
      if(pw == "anatime"):    eline.append(anatime);
      if(pw == "error"):      eline.append(errorstatus);
      if(pw == "isrunning"):  eline.append(isrunning);
      if(pw == "xmldir"):     eline.append(xmldir);
      if(pw == "freesurfer"):
        subject = BIRNID2FreeSurfer(bid,fsbirnid);
        if(len(subject) == 0): subject = "notfound";
        eline.append(subject);
      #endif
    #end PrintWhat
    print fmt % tuple(eline);
  #end Loop over DB

  return 0;
#end def

#-------------------------------------------------------------
def CheckPrintWhat(PrintWhat):
  """
  err = CheckPrintWhat(PrintWhat);

  """
  for pw in PrintWhat:
    err = 1;
    if(pw == "row"):        err = 0;
    if(pw == "birnid"):     err = 0;
    if(pw == "recsite"):    err = 0;
    if(pw == "acqsite"):    err = 0;
    if(pw == "project"):    err = 0;
    if(pw == "visit"):      err = 0;
    if(pw == "visitdate"):  err = 0;
    if(pw == "study"):      err = 0;
    if(pw == "studyid"):    err = 0;
    if(pw == "paradigm"):   err = 0;
    if(pw == "run"):        err = 0;
    if(pw == "analevel"):   err = 0;
    if(pw == "ananame"):    err = 0;
    if(pw == "anadate"):    err = 0;
    if(pw == "anatime"):    err = 0;
    if(pw == "error"):      err = 0;
    if(pw == "isrunning"):  err = 0;
    if(pw == "xmldir"):     err = 0;
    if(pw == "freesurfer"): err = 0;
    if(err):
      print "ERROR: %s unrecognized" % pw;
      return 1;
    #endif
  #endfor
  return 0;
#enddef

#-------------------------------------
def DB2HTML(db,PrintWhat,fsbirnid):

  html = "<html>\n";
  html = html + "<TITLE> FIPS Database </TITLE>\n";
  html = html +"<center><h1> FIPS Database </h1></center>\n";
  html = html +"For more information about FIPS and fBIRN see the FIPS <a href=http://xwiki.nbirn.net:8080/xwiki/bin/view/Project-FIPS-Public/Home> Wiki.</a>\n";
  html = html +  "<br><br><br>";
    
  proclist = fips.LoadProcs();
  if(len(proclist)>0):
    html = html +"<table cellpadding=2 cellspacing=2 border=1 style=text-align: left; width: 100%;>\n";
    html = html +"<CAPTION>Currently Running FIPS Processes</CAPTION>"
    html = html +"<tbody>\n";
    html = html +"<tr>\n";
    html = html +"<td>CMD</td> <td>User</td> <td>Host</td> <td>ProcId</td> <td>Date</td> <td>Time</td>\n";
    html = html +"</tr>\n";
    for proc in proclist:
      (procdate,proctime,user,host,procid,cwd,cmd,logfile) = proc;
      html = html + "<td><a href = \"file:%s\"> %s </a> </td> <td>%s</td> <td>%s</td> <td>%s</td> <td>%s</td> <td>%s</td>" % (logfile,cmd,user,host,procid,procdate,proctime);
      html = html + "</tr>\n";
    #endfor
    html = html + "</tbody>";
    html = html +  "</table>";
    html = html +  "<br><br><br>";
  #endif    

  html = html +"<table cellpadding=2 cellspacing=2 border=1 style=text-align: left; width: 100%;>\n";
  html = html +"<CAPTION>Datasets found in  FIPS Database</CAPTION>"
  html = html +"<tbody>\n";
  html = html +"<tr>\n";
  html = html +"<td> Nth </td>\n";
  for pw in PrintWhat:
    if(pw == "project"): html = html +"<td> Project </td>\n";
    if(pw == "birnid"): html = html +"<td> BIRNId </td>\n";
    if(pw == "acqsite"): html = html +"<td> Site </td>\n";
    if(pw == "visit"): html = html +"<td> Visit </td>\n";
    if(pw == "visitdate"): html = html +"<td> Date </td>\n";
    if(pw == "paradigm"): html = html +"<td> Paradigm </td>\n";
    if(pw == "run"): html = html +"<td> Run </td>\n";
    if(pw == "analevel"): html = html +"<td> Level </td>\n";
    if(pw == "ananame"): html = html +"<td> Analysis </td>\n";
    if(pw == "anadate"): html = html +"<td> Ana Date </td>\n";
    if(pw == "anatime"): html = html +"<td> Ana Time </td>\n";
    if(pw == "error"): html = html +"<td> Error </td>\n";
    if(pw == "isrunning"): html = html +"<td> Running </td>\n";
    if(pw == "freesurfer"): html = html +"<td> FreeSurfer </td>\n";
  #endfor
  html = html +"<td> Path </td>\n";
  html = html +"</tr>\n";
  mth = 0;
  bid_prev = '';
  acqsite_prev = '';
  visitid_prev = -1;
  color = "#009900";
  for entry in db:
    mth = mth + 1;
    (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
     visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
     anatime,errorstatus,isrunning,xmldir) = entry;
    #if(mth == 1): bid_prev = bid;
    if(bid != bid_prev or acqsite != acqsite_prev or visitid != visitid_prev):
      if(color == "#009900"):
        color = "#000000";
      else:
        color = "#009900";
      bid_prev = bid;
      acqsite_prev = acqsite;
      visitid_prev = visitid;
    #endif
    if(errorstatus):
      coloruse = "#FF0000";
    else:
      coloruse = color;
    if(analevel == "none"):
      analevel = "Raw";
      ananame = "None";
    #endif
    html = html + "\n";
    html = html + "<tr>";
    html = html + "<td> %2d </td>" % (mth);
    for pw in PrintWhat:
      if(pw == "project"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,projname);
      if(pw == "birnid"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,bid);
      if(pw == "acqsite"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,acqsite);
      if(pw == "visit"): html = html + "<td> <font color=\"%s\"> %d </font> </td>" % (coloruse,visitid);
      if(pw == "visitdate"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,visitdate);
      if(pw == "paradigm"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,paradigm);
      if(pw == "run"): html = html + "<td> <font color=\"%s\"> %d </font> </td>" % (coloruse,seriesnum);
      if(pw == "analevel"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,analevel);
      if(pw == "ananame"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,ananame);
      if(pw == "anadate"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,anadate);
      if(pw == "anatime"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,anatime);
      if(pw == "error"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,errorstatus);
      if(pw == "isrunning"): html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,isrunning);
      if(pw == "freesurfer"):
        subject = BIRNID2FreeSurfer(bid,fsbirnid);
        if(len(subject) == 0): subject = "notfound";
        html = html + "<td> <font color=\"%s\"> %s </font> </td>" % (coloruse,subject);
      #endif
    #endfor
    report = xmldir + "/report_poststats.html";
    if(not os.path.exists(report)): report = xmldir + "/report.html";
    if(os.path.exists(report)): 
      html = html + "<td> <a href = \"file:%s\"> report </a> </td>" % (report);
    else:
      html = html + "<td> <a href=\"file:%s\"> dir </a> </td>" % (xmldir);
      #html = html + "<td> <a href=\"file:%s\"> <font color=\"#FF0000\"> dir </font> </a> </td>" % (xmldir);
    #endif
    html = html + "</tr>\n";

    #xmlfile = xmldir + "/fips-process.xml";

  #end for    

  html = html + "</tbody>";
  html = html +  "</table>";
  html = html +  "</html>";

  return html;
#enddef




#-------------------------------------
def UpdateSQLiteDB(xmlfile):
  (bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
   visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
   anatime,errorstatus,isrunning,xmldir) = XML2DBEntry(xmlfile);

  #tmpfile = os.path.join("/tmp",tempfile.mktemp());
  tmpfile = "mysqlpycmd";
  fp = open(tmpfile,'w');
  fp.write(".timeout 100000 \n");
  fp.write("UPDATE fips\n");
  fp.write("SET\n");
  fp.write("recruite_site  = \"%s\", \n" % recsite);
  fp.write("visit_date     = \"%s\", \n" % visitdate);
  fp.write("series_type    = \"%s\", \n" % seriestype,);
  fp.write("analysis_level = \"%s\", \n" % analevel);
  fp.write("analysis_date = \"%s\", \n" % anadate);
  fp.write("analysis_time = \"%s\", \n" % anatime);
  fp.write("error_status  = %d, \n" % errorstatus);
  fp.write("is_running    = %d, \n" % isrunning);
  fp.write("xml_dir       = \"%s\"\n" % xmldir);
  fp.write("WHERE\n");
  fp.write("birnid        = \"%s\" AND \n" % bid);
  fp.write("acq_site      = \"%s\" AND \n" % acqsite);
  fp.write("project       = \"%s\" AND \n" % projname);
  fp.write("study_name    = \"%s\" AND \n" % studyname);
  fp.write("study_id      = \"%s\" AND \n" % studyid);
  fp.write("visit_name    = \"%s\" AND \n" % visitname);
  fp.write("visit_id      = \"%s\" AND \n" % visitid);
  fp.write("paradigm      = \"%s\" AND \n" % paradigm);
  fp.write("series_number = \"%s\" AND \n" % seriesnum);
  fp.write("analysis_name = \"%s\";\n" %     ananame);
  fp.close();

  dbdir = os.environ.get('FIPS_DB_DIR',0);
  sqldbfile = os.path.join(dbdir,"fips.sqlite.dat");
  cmd = "sqlite3 %s \".read %s\" " % (sqldbfile,tmpfile);
  print cmd;
  fp = os.popen(cmd,'r');
  tmp = fp.read();
  if(len(tmp) > 0):
    print tmp;
  #endif

#end  

#-------------------------------------------------------------
def DBMatchNthVisit(db,nthvisit):
  """
  DBMatchNthVisit(db,nthvisit). Match by the ordinal visit.
  """
  vdb = [];
  # Get a list unique bid-site combos
  ubs = [];
  for entry in db:
    (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
     visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
     anatime,errorstatus,isrunning,xmldir) = entry;
    hit = 0;
    for bs in ubs:
      if(bid == bs[0] and acqsite == bs[1]):
        hit = 1;
      #endif
    #endfor
    if(not hit):
      ubs.append((bid,acqsite));
    #endif
  #end

  bsvlist = [];
  for bs in ubs:

    # Get a list of all the visits assoc with this bs
    vlist = [];
    for entry in db:
      (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
       visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
       anatime,errorstatus,isrunning,xmldir) = entry;
      if(bid == bs[0] and acqsite == bs[1]):
        vlist.append(visitid);
      #endif
    #endfor

    # For list of all visits, get a list of unique visits
    uvlist = [];
    for v in vlist:
      hit = 0;
      for uv in uvlist:
        if(v == uv):
          hit = 1;
        #endif
      #endfor
      if(not hit):
        uvlist.append(v);
      #endif
    #endfor

    # Sort unique list in ascending order
    uvlist.sort();

    # Extract the nthvisit number 
    if(len(uvlist) >= nthvisit):
      bsvlist.append((bs[0],bs[1],uvlist[nthvisit-1]));
    #endif

  #endfor

  for entry in db:
    (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
     visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
     anatime,errorstatus,isrunning,xmldir) = entry;
    hit = 0;
    for bsv in bsvlist:
      if(bid == bsv[0] and acqsite == bsv[1] and visitid == bsv[2]):
        hit = 1;
      #endif
    #endfor
    if(hit):
      vdb.append(entry);
    #endif
  #end

  return vdb;

#enddef

#---------------------------------------------------------
def SortByRun(db,nruns):

  dbsorted = [];

  # Get a unique list of bid-site-visit combos
  ubsvlist = [];
  for entry in db:
    (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
     visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
     anatime,errorstatus,isrunning,xmldir) = entry;
    if(analevel != 'FirstLevel' and analevel != 'none'):
      dbsorted.append(entry);
      continue;
    #endif
    hit = 0;
    for ubsv in ubsvlist:
      if(bid == ubsv[0] and acqsite == ubsv[1] and visitid == ubsv[2]):
        hit = 1;
      #endif
    #endfor
    if(not hit):
      ubsvlist.append((bid,acqsite,visitid));
    #endif
  #end for    

  for ubsv in ubsvlist:
    ubsv_entries = [];
    runlist = [];
    for entry in db:
      (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
       visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
       anatime,errorstatus,isrunning,xmldir) = entry;
      if(analevel != 'FirstLevel' and analevel != 'none'):
        continue;
      #endif
      if(bid == ubsv[0] and acqsite == ubsv[1] and visitid == ubsv[2]):
        ubsv_entries.append(entry);
        runlist.append(seriesnum);
      #endif
    #endfor entry

    # Sort run list in ascending order
    runlist.sort();
    if(nruns > 0):
      if(len(runlist) < nruns):
        continue;
      #endif
      runlist = runlist[0:nruns];
    #endif

    for run in runlist:
      for entry in ubsv_entries:
        (nth,bid,recsite,acqsite,projname,studyname,studyid,visitname,visitid,
         visitdate,seriestype,paradigm,seriesnum,analevel,ananame,anadate,
         anatime,errorstatus,isrunning,xmldir) = entry;
        if(run == seriesnum):
          dbsorted.append(entry);
        #endif
      #endfor entry
    #endfor run
  #endfor ubsv

  return dbsorted;

#end def SortByRun(db):


#---------------------------------------------------------------------
def LoadDB(*argv):
  """
  db = LoadDB(<dbfile>,<proj>,<bidlist>);
  """
  db = [];
  dbfile = [];
  proj = [];
  bidlist = [];
  
  ntharg = 0;
  for arg in argv:
    if(ntharg == 0): dbfile  = argv[ntharg];
    if(ntharg == 1): proj    = argv[ntharg];
    if(ntharg == 2): bidlist = argv[ntharg];
    ntharg = ntharg + 1;
  #end for 

  usepb = os.environ.get('FIPS_DB_PROJ_BID',1);
  if(not usepb or len(argv) == 1):
    db = LoadDB0(dbfile);
    return db;
  #end

  dbdir = os.environ.get('FIPS_DB_DIR',0);
  dbfile0 = os.path.join(dbdir,"fips.database.dat");

  dbflist = FindDB(dbdir);
  db = [];
  for dbf in dbflist:
    if(dbf == dbfile0): continue;
    a = string.split(dbf,'/');
    na = len(a);
    if(na < 4):
      print "ERROR: parsing DB PBP path %s" % (dbf);
      print a;
      return [];
    #end
    dbfproj = a[na-3];
    dbfbid  = a[na-2];
    if(len(proj) > 0 and proj != dbfproj): continue
    if(len(bidlist) > 0):
      ok = 0;
      for bid in bidlist:
        if(bid == dbfbid): ok = 1;
      #end for
      if(not ok): continue
    #endif
    #print "Loading %s" % (dbf);
    db0 = LoadDB0(dbf);
    db = MergeDB(db,db0);
  #end
  return db;
#end def LoadDB(*argv):

#---------------------------------------------------------------------
def LoadDB0(*dbfile):
  """
  db = LoadDB0(<dbfile>);
  Loads db as an Nx20 matrix. If dbfile not spec, then uses default.
    Eg, print db[25][19];
  """
  db = [];
  if(len(dbfile) == 0):
    dbfile = ();
  else:
    dbfile = dbfile[0];
  if(len(dbfile) == 0): 
    dbdir = os.environ.get('FIPS_DB_DIR',0);
    dbfile = os.path.join(dbdir,"fips.database.dat");
  #endif    
  if(not os.path.exists(dbfile)):
    print "ERROR: cannot find dbfile %s" % dbfile;
    sys.exit(1);
  #endif

  #print "LoadDB(): Loading %s" % (dbfile);

  fp = open(dbfile,'r');
  nth = 0;
  for line in fp:
    #print "%s" % line;
    ind = line.rfind('#');
    if(ind != -1): continue;
    tmp = string.split(line);
    # Convert some from strings to actual numbers
    tmp[8] = string.atoi(tmp[8]);   # visit number
    tmp[12] = string.atoi(tmp[12]); # run number
    tmp[17] = string.atoi(tmp[17]); # error status
    tmp[18] = string.atoi(tmp[18]); # isrunning
    db.append(tmp);
    nth = nth + 1;
  #end for line
  fp.close();
  return db;
#end: def LoadDB(dbfile)

#---------------------------------------------------------------------
def DBLock(dblockfile):
  """
  err = DBLock(dblockfile)
  """
  try:
    fp = open(dblockfile,'w');
  except:
    print "ERROR: could not open %s" % dblockfile;
    return 1;
  #endtry

  fp.write("FIPS DB Lockfile\n");
  try:
    user = os.getlogin();
  except:
    user = "unknown";
  #endtry
  fp.write("USER %s\n" % user );
  fp.write("DATE %s\n" % datetime.datetime.now());
  fp.write("UNAME %s\n" % os.uname()[1]);
  fp.write("TIMESTAMP %f\n" % time.time());
  fp.write("PID %d\n" % os.getpid());
  fp.close();
  os.chmod(dblockfile, 0666); # User and Group read/write
  return 0;
#endif def

#---------------------------------------------------------------------
def DBUnlock(dbfile):
  dblockfile = dbfile + '.lock';
  if(os.path.exists(dblockfile)):
    os.remove(dblockfile);
    try:
      user = os.getlogin();
    except:
      user = "unknown";
    #endtry
    (luser,ldate,ltime,lhost,tlocked) = ReadDBLock(dblockfile);
    tlockedsec = 60*tlocked;
    dbdir = os.environ.get('FIPS_DB_DIR',0);
    lockmsg = os.path.join(dbdir,"lock.db.msg");
    try:
      fp = open(lockmsg,'a');
    except:
      print "ERROR: could not open %s" % lockmsg;
      return;
    #endtry
    fp.write("%s %s %s %s Unlocking %4.1f\n" %
             (datetime.datetime.now(),user,os.uname()[1],os.getpid(),tlockedsec));
    fp.close();
    os.chmod(lockmsg, 0666); # User and Group read/write
  #endif
  return;
#endif def
  
#---------------------------------------------------------------------
def WaitForDBLock(dbfile):
  """
  TimedOut = WaitForDBLock(dbfile)
  Waits for dbfile.lock to disappear, then creates it. It it
  waits for 300 sec, then times out. Waits 1 sec between tries.
  """
  try:
    user = os.getlogin();
  except:
    user = "unknown";
  #endtry
  dbdir = os.environ.get('FIPS_DB_DIR',0);
  lockmsg = os.path.join(dbdir,"lock.db.msg");
  dblockfile = dbfile + '.lock';
  nth = 0;
  while(os.path.exists(dblockfile) and nth < 300):
    (luser,ldate,ltime,lhost,tlocked) = ReadDBLock(dblockfile);
    nth = nth + 1;
    try:
      fp = open(lockmsg,'a');
    except:
      print "ERROR: could not open %s" % lockmsg;
      return 2;
    #endtry
    fp.write("%s %s %s %s Waiting %d %4.1f " %
             (datetime.datetime.now(), user,os.uname()[1],
              os.getpid(),nth,tlocked));
    if(tlocked > 10):
      fp.write("WARNING: lock is more than 10 min old\n");
      print "WARNING: lock is more than 10 min old";
    else:
      fp.write("\n");
    fp.close();
    os.chmod(lockmsg, 0666); # All read/write    
    print("%s %s %s %s Waiting %d %4.1f " %
          (datetime.datetime.now(), user,os.uname()[1],
           os.getpid(),nth,tlocked));
    time.sleep(1);
  #end while
  if(os.path.exists(dblockfile)):
    (luser,ldate,ltime,lhost,tlocked) = ReadDBLock(dblockfile);
    try:
      fp = open(lockmsg,'a');
    except:
      print "ERROR: could not open %s" % lockmsg;
      return 2;
    #endtry
    fp.write("%s %s %s %s TimedOut %4.1f\n" %
             (datetime.datetime.now(), user, os.uname()[1], os.getpid(), tlocked));
    fp.close();
    os.chmod(lockmsg, 0666); # All read/write    
    print "ERROR: timed out wating for %s for %4.1f min" % (dblockfile,tlocked);
    return 1;
  #endif
  try:
    fp = open(lockmsg,'a');
  except:
    print "ERROR: opening %s" % lockmsg;
    return 2;
  #end try    
  fp.write("%s %s %s %s Locking\n" %
           (datetime.datetime.now(),user,os.uname()[1],os.getpid()));
  fp.close();
  os.chmod(lockmsg, 0666); # All read/write    
  err = fips.DBLock(dblockfile);
  return err;
#end: def WaitForDBLock(dbfile)

#---------------------------------------------------------------------
def ReadDBLock(dblockfile):
  try:
    fp = open(dblockfile,'r');
  except:
    return ('null','null','null','null',-1);
  txt = fp.read();
  fp.close();
  items = txt.split();
  luser = items[4];
  ldate = items[6];
  ltime = items[7];
  luname = items[9];
  timestamp = string.atof(items[11]);
  tlockedmin = (time.time() - timestamp)/60.0;
  return (luser,ldate,ltime,luname,tlockedmin);
#end def  

#---------------------------------------------------------------------
def WriteDB(db,dbfile):
  """
  err = WriteDB(db,dbfile)
  Writes a DB file including preamble. db is Nx20. Renumbers 1st column.
  """
  usepb = os.environ.get('FIPS_DB_PROJ_BID',1);
  if(not usepb):
    r = WriteDB0(db,dbfile);
    return r;
  #end

  dbdir = os.environ.get('FIPS_DB_DIR',0);

  projlist = UniqueProjects(db);
  for proj in projlist:
    bidlist = UniqueBIRNIDs(db,proj,'');
    if(len(bidlist) == 0): continue;
    for bid in bidlist:
      match = ['','','','','','','','',[],'','','',[],'','','','',-1,-1,''];
      match[1] = bid;
      match[4] = proj;
      dbmatch = fips.DBMatch(match,db);
      if(len(dbmatch) == 0): continue;
      dbfile = os.path.join(dbdir,proj,bid,'fips.database.dat');
      r = WriteDB0(dbmatch,dbfile);
      if(r != 0): return r;
    #end bid
  #end proj

  return 0;
#end def WriteDB(db,dbfile):


#---------------------------------------------------------------------
def WriteDB0(db,dbfile):
  """
  err = WriteDB(db,dbfile)
  Writes a DB file including preamble. db is Nx20. Renumbers 1st column.
  """
  # Make the directory
  dbdir = os.path.dirname(dbfile);
  if(dbdir == ''):
    dbdir = '.';
  #endif
  err = fips.mkdirp(dbdir);
  if(err): return 1;
  try:
    fp = open(dbfile,"w");
  except:
    print "ERROR: could not open %s for writing" % (dbfile);
    return 1;
  #end try    
  fips.PrintDBPreamble(fp);
  nth = 1;
  for entry in db:
    #pdb.set_trace();
    entry[0] = nth;
    fmt = "%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n";
    e = fmt % tuple(entry);
    fp.write(e);
    #print "%s" % e;
    nth = nth + 1;
  #endfor
  fp.close();
  os.chmod(dbfile, 0660); # User and Group read/write
  return 0;
#def WriteDB(db,dbfile)

#---------------------------------------------------------------------
def LoadDBSQLite(*sqlitefile):

  db = [];
  
  if(len(sqlitefile) == 0):
    sqlitefile = ();
  else:
    sqlitefile = sqlitefile[0];
  if(len(sqlitefile) == 0): 
    dbdir = os.environ.get('FIPS_DB_DIR',0);
    sqlitefile = os.path.join(dbdir,"fips.database.sqlite");
  #endif    

  if(not os.path.exists(sqlitefile)):
    print "ERROR: cannot find %s" % sqlitefile;
    sys.exit(1);
  #endif

  conn = sqlite.connect(sqlitefile);
  c = conn.cursor()
  c.execute("SELECT * from fipsdb");
  db = c.fetchall();
  return db;

#end def LoadDBSQLite(*sqlitefile):

#---------------------------------------------------------------------
def WriteDBSQLite(db,*sqlitefile):

  if(len(sqlitefile) == 0):
    sqlitefile = ();
  else:
    sqlitefile = sqlitefile[0];
  if(len(sqlitefile) == 0): 
    dbdir = os.environ.get('FIPS_DB_DIR',0);
    sqlitefile = os.path.join(dbdir,"fips.database.sqlite");
  #endif    

  # delete it if it exists
  if(os.path.exists(sqlitefile)): os.remove(sqlitefile);

  conn = sqlite.connect(sqlitefile);
  c = conn.cursor()

  # Create table
  c.execute('''create table fipsdb (
    ROWID            INTEGER PRIMARY KEY AUTOINCREMENT,
    BIRNID           VARCHAR(20),
    RECRUITE_SITE    VARCHAR(20),
    ACQ_SITE         VARCHAR(20),
    PROJECT          VARCHAR(20),
    STUDY_NAME       VARCHAR(20),
    STUDY_ID         VARCHAR(20),
    VISIT_NAME       VARCHAR(20),
    VISIT_ID         INTEGER,
    VISIT_DATE       VARCHAR(20),
    SERIES_TYPE      VARCHAR(20),
    PARADIGM         VARCHAR(20),
    SERIES_NUMBER    INTEGER,
    ANALYSIS_LEVEL_1 VARCHAR(20),
    ANALYSIS_NAME    VARCHAR(20),
    ANALYSIS_DATE    DATE,
    ANALYSIS_TIME    TIME,
    ERROR_STATUS     VARCHAR(20),
    IS_RUNNING       BOOLEAN,
    XML_DIR          VARCHAR(20))''')

  for e in db:
    c.execute("insert into fipsdb values (null,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
              tuple(e[1:]));
  #endfor    
  conn.commit();
  c.close();

#end  def DB2SQLite(db,sqlitefile):

#---------------------------------------------------------------------
def PruneDB(db):
  """
  db = PruneDB(db); Removes entries that do not exist. Renumbers 1st
  column.
  """
  print "Pruning, starting with %d" % len(db);
  dbpruned = [];
  nth = 1;
  for entry in db:
    #pdb.set_trace();
    xmldir = entry[len(entry)-1];
    xmlfile = os.path.join(xmldir,"fips-process.xml");
    if(os.path.exists(xmlfile)):
      entry[0] = nth;
      dbpruned.append(entry);
      nth = nth + 1;
    #endif
  #endfor
  print "Pruning, done with %d" % len(db);
  return dbpruned;
#def PruneDB(db)

#---------------------------------------------------------------------
def SynchDB(db):
  """
  db = SynchDB(db);
  Synchronizes DB.  Reloads entries from xmlfiles. Removes entries
  that do not exist.  Removes duplicates. Renumbers 1st column. Does 
  not search for new xml files (so faster than doing a full rebuild).
  """
  print "Synch, starting with %d" % len(db);
  dbval = [];
  nth = 1;
  for entry in db:
    #pdb.set_trace();
    xmldir = entry[len(entry)-1];
    xmlfile = os.path.join(xmldir, "fips-process.xml");
    if(os.path.exists(xmlfile)):
      entry = list(XML2DBEntry(xmlfile));
      entry.insert(0,nth);
      dbval.append(entry);
      nth = nth + 1;
    #endif
  #endfor
  dbval = fips.HUniqueDB(dbval);
  print "Synch, done with %d" % len(dbval);
  return dbval;
#def SynchDB(db)

#----------------------------------------------------------------
def DBSort(db,SortOrder):

  dbsorted = [];
  dbsorted.append(db[0]);
  nentries = len(db);
  
  for entry in db[1:]:
    nth = 0;
    added = 0;
    for esorted in dbsorted:
      r = CompareDBEntries(entry,esorted,SortOrder);
      if(r <= 0):
        dbsorted.insert(nth,entry);
        added = 1;
        break;
      #endif
      nth = nth + 1;
    #end
    if(not added): dbsorted.append(entry);
  #end

  return dbsorted;

#end def


#--------------------------------------------------------------------------------
def CompareDBEntries(entry1,entry2,CompareOrder):

  (nth1,bid1,recsite1,acqsite1,projname1,studyname1,studyid1,visitname1,visit1,
   visitdate1,seriestype1,paradigm1,run1,analevel1,ananame1,anadate1,
   anatime1,errorstatus1,isrunning1,xmldir) = entry1;

  (nth2,bid2,recsite2,acqsite2,projname2,studyname2,studyid2,visitname2,visit2,
   visitdate2,seriestype2,paradigm2,run2,analevel2,ananame2,anadate2,
   anatime2,errorstatus2,isrunning2,xmldir) = entry2;

  a = 0;
  b = 0;
  for key in CompareOrder:
    if(key == "project"):   a=projname1; b=projname2;
    if(key == "birnid"):    a=bid1; b=bid2;
    if(key == "acqsite"):   a=acqsite1; b=acqsite2;
    if(key == "visit"):     a=visit1; b=visit2;
    if(key == "visitdate"): a=visitdate1; b=visitdate2;
    if(key == "paradigm"):  a=paradigm1; b=paradigm2;
    if(key == "run"):       a=run1; b=run2;
    if(key == "analevel"):
      if(analevel1 != analevel2):
        if(analevel1 == "none"): return -1;
        if(analevel2 == "none"): return +1;
        if(analevel1 == "PreProc"): return -1;
        if(analevel2 == "PreProc"): return +1;
        if(analevel1 == "FirstLevel"): return -1;
        if(analevel2 == "FirstLevel"): return +1;
      #endif
      a=analevel1; b=analevel2;
    #endif
    if(key == "ananame"):   a=ananame1; b=ananame2;
    if(key == "anadate"):   a=anadate1; b=anadate2;
    if(key == "anatime"):   a=anatime1; b=anatime2;
    if(key == "error"):     a=errorstatus1; b=errorstatus2;
    if(key == "isrunning"): a=isrunning1; b=isrunning2;
    if(a < b): return -1;
    if(a > b): return +1;
  #end

  return 0;
#end def

#----------------------------------------------------------
def LoadProcs(*procdir):
  if(len(procdir) == 0): procdir = ();
  if(len(procdir) == 0): 
    procdir = os.environ.get('FIPS_PROC_DIR',0);
  #endif    
  proclist = [];
  print procdir;
  for (root, dirs, files) in os.walk(procdir):
    for f in files:
      proc = fips.LoadProcFile(os.path.join(procdir,f));
      if(len(proc) == 0): continue;
      proclist.append(proc);
    #end for f
  #end for walk
  return proclist;
#end def LoadProcs(*procdir):

#----------------------------------------------------------
def LoadProcFile(procfile):
  # fipsproc = (procdate,proctime,user,host,procid,cwd,cmd,logfile);

  fp = open(procfile,"r");
  line = fp.readline();
  ind = line.rfind('FIPS-PROCESS-CONTROL-FILE');  
  if(ind == -1): return [];
    
  procdate = [];
  proctime = [];
  user = [];
  host = [];
  procid = [];
  cwd = [];
  cmd = [];
  logfile = [];
  for line in fp:
    ind = line.rfind('FIPSDATE');
    if(ind != -1): 
      tmp = string.split(line);
      procdate = string.join(tmp[1:4]);
      proctime = tmp[4];
      continue;
    #endif
    ind = line.rfind('FIPSUSER');
    if(ind != -1): 
      tmp = string.split(line);
      user = tmp[1];
      continue;
    #endif
    ind = line.rfind('FIPSHOSTNAME');
    if(ind != -1): 
      tmp = string.split(line);
      host = tmp[1];
      continue;
    #endif
    ind = line.rfind('FIPSPROCID');
    if(ind != -1): 
      tmp = string.split(line);
      procid = tmp[1];
      continue;
    #endif
    ind = line.rfind('FIPSCWD');
    if(ind != -1): 
      tmp = string.split(line);
      cwd = tmp[1];
      continue;
    #endif
    ind = line.rfind('FIPSCOMMAND');
    if(ind != -1): 
      tmp = string.split(line);
      cmd = tmp[1];
      continue;
    #endif
    ind = line.rfind('FIPSLOGFILE');
    if(ind != -1): 
      tmp = string.split(line);
      logfile = tmp[1];
      continue;
    #endif

  #end for

  fipsproc = (procdate,proctime,user,host,procid,cwd,cmd,logfile);
  return fipsproc;

#end def LoadProcFile(procfile)

#------------------------------------------------------------
def LoadInstance(*tupInstanceFile):
  """
  LoadInstance(*tupInstanceFile)
  Loads a FIPS instance file. This is a file with csh-style
  setenv commands in it. For each setenv, it creates an environment
  variable in the python context and sets it to the given
  value. 
  """

  if(len(tupInstanceFile) == 0):
    # If FIPS_INSTANCE env var does not exist, just return
    try:
      InstanceFile = os.environ["FIPS_INSTANCE"];
    except:
      return;
  else:
    InstanceFile = tupInstanceFile[0];
  #endif
  #print "Instance File is %s" % InstanceFile;
  if(not os.path.exists(InstanceFile)):
    print "ERROR: cannot find instance %s" % InstanceFile;
    sys.exit(1);
  #endif
  try:
    fp = open(InstanceFile,'r');
  except:
    print "ERROR: opening instance file %s" % InstanceFile;
    sys.exit(1);
  #endtry
  for line in fp:
    ind = line.rfind('setenv');
    if(ind == -1): continue ;
    items = line.split();
    d = items[2].find("$");
    if(d != -1):
      print "ERROR: variable %s in instancce %s has $ (%s)" % (items[1],InstanceFile,items[2]);
      sys.exit(1);
    #endif
    os.environ[items[1]] = os.path.expanduser(items[2]);
    #print "setenv %s %s" % (items[1],items[2]);
  #endfor

  return ;
#------------------------------------------------------------
