function [qamcstruct] = LoadQAMCSum(mcsumfile)
% qamcstruct = LoadQAMCSum(mcsumfile)
%
% Loads in the file created with fips-qa-sum. Eg:
%   fips-qa-sum --mc mc.dat --flac fb3-wm-sm5
%
% qastruct has the following members:
%  qatable 
%  birnids - N-by-1 string array of birnids 
%  visitdates - N-by-1 string array of  YYYY-MM-DD
%  visitdatenums - N-by-1 array of the visit date 
%     expressed as the number of days since Jan 1 2007.
%  acqsites - N-by-1 string array of acquisition sites
%  acqsitenos - N-by-1 array of acquisition sites numbers
%  visitnos - N-by-1 vector of visit numbers
%  runnos - N-by-1 vector of run numbers
%  measnames - 2-by-M string array of the 2 MC measures
% 
% qatable is N-by-10
% 1.  mcabs
% 2.  mcrel
%
% $Id: LoadQAMCSum.m,v 1.1 2007/10/09 21:10:53 dgreve Exp $

qamcstruct = [];

if(nargin ~= 1)
  fprintf('qamcstruct = LoadQAMCSum(mcsumfile)\n');
  return
end

measnames = '';
measnames = strvcat(measnames,'MC Abs');
measnames = strvcat(measnames,'MC Rel');

fp = fopen(mcsumfile,'r');
if(fp == -1)
  fprintf('ERROR: could not open %s\n',mcsumfile);
  return;
end

birnids = [];
visitdates = [];
acqsites = [];
acqsitenos = [];
visitnos = [];
runnos = [];
qatable = [];
vdatenums = [];

% Serial date number of Jan 1 2007 to use as a baseline
vdatenum0 = datenum(2007,1,1);

tic;
nrows = 0;
while(1)
  nrows = nrows + 1;

  fscanf(fp,'%s',1); % past nth
  [bid   count] = fscanf(fp,'%s',1);
  if(count ~= 1) break; end
  [vyear  count]   = fscanf(fp,'%d',1);
  [vmonth count]   = fscanf(fp,'%d',1);
  [vday   count]   = fscanf(fp,'%d',1);
  [acqsite count] = fscanf(fp,'%s',1);
  [vno count]     = fscanf(fp,'%d',1);
  [rno count]     = fscanf(fp,'%d',1);
  [qaline count]  = fscanf(fp,'%f',3);
  
  birnids = strvcat(birnids,bid);
  vdate = sprintf('%4d-%02d-%02d',vyear,vmonth,vday);  
  visitdates = strvcat(visitdates,vdate);
  acqsites   = strvcat(acqsites,acqsite);
  %acqsiteno = sscanf(acqsite,'%d');
  %acqsitenos = [acqsitenos; acqsiteno];
  visitnos = [visitnos; vno];
  runnos = [runnos; rno];
  
  qatable = [qatable; qaline'];

  % This computes the serial date number for the visit
  % relative to Jan 1 2007
  %ind = find(vdate == '-');
  %vdate(ind) = ' ';
  %^ymd = sscanf(vdate,'%d %d %d');

  ymd = [vyear vmonth vday];  
  vdatenum = datenum(ymd(1),ymd(2),ymd(3)) - vdatenum0;
  vdatenums = [vdatenums; vdatenum];
  
end
%fprintf('%4d %g\n',nrows,toc);

fclose(fp);

% Sort by Acq Date
[a ii] = sort(vdatenums);

qamcstruct.qatable = qatable(ii,:);
qamcstruct.birnids = birnids(ii,:);
qamcstruct.visitdates = visitdates(ii,:);
qamcstruct.visitdatenums = vdatenums(ii,:);
qamcstruct.acqsites = acqsites(ii,:);
qamcstruct.visitnos = visitnos(ii,:);
qamcstruct.runnos = runnos(ii,:);
qamcstruct.measnames = measnames;
qamcstruct.mcabs = qamcstruct.qatable(:,1);
qamcstruct.mcrel = qamcstruct.qatable(:,2);
qamcstruct.mcrelstd = qamcstruct.qatable(:,3);
%qamcstruct.acqsitenos = acqsitenos(ii,:);

return;
